"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ObservationUI(google.protobuf.message.Message):
    """
    Observation
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GROUPS_FIELD_NUMBER: builtins.int
    SINGLE_FIELD_NUMBER: builtins.int
    MULTI_FIELD_NUMBER: builtins.int
    CARGO_FIELD_NUMBER: builtins.int
    PRODUCTION_FIELD_NUMBER: builtins.int
    @property
    def groups(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ControlGroup]: ...
    @property
    def single(self) -> global___SinglePanel: ...
    @property
    def multi(self) -> global___MultiPanel: ...
    @property
    def cargo(self) -> global___CargoPanel: ...
    @property
    def production(self) -> global___ProductionPanel: ...
    def __init__(
        self,
        *,
        groups: collections.abc.Iterable[global___ControlGroup] | None = ...,
        single: global___SinglePanel | None = ...,
        multi: global___MultiPanel | None = ...,
        cargo: global___CargoPanel | None = ...,
        production: global___ProductionPanel | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cargo", b"cargo", "multi", b"multi", "panel", b"panel", "production", b"production", "single", b"single"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cargo", b"cargo", "groups", b"groups", "multi", b"multi", "panel", b"panel", "production", b"production", "single", b"single"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["panel", b"panel"]) -> typing_extensions.Literal["single", "multi", "cargo", "production"] | None: ...

global___ObservationUI = ObservationUI

@typing_extensions.final
class ControlGroup(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTROL_GROUP_INDEX_FIELD_NUMBER: builtins.int
    LEADER_UNIT_TYPE_FIELD_NUMBER: builtins.int
    COUNT_FIELD_NUMBER: builtins.int
    control_group_index: builtins.int
    leader_unit_type: builtins.int
    count: builtins.int
    def __init__(
        self,
        *,
        control_group_index: builtins.int | None = ...,
        leader_unit_type: builtins.int | None = ...,
        count: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["control_group_index", b"control_group_index", "count", b"count", "leader_unit_type", b"leader_unit_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["control_group_index", b"control_group_index", "count", b"count", "leader_unit_type", b"leader_unit_type"]) -> None: ...

global___ControlGroup = ControlGroup

@typing_extensions.final
class UnitInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_TYPE_FIELD_NUMBER: builtins.int
    PLAYER_RELATIVE_FIELD_NUMBER: builtins.int
    HEALTH_FIELD_NUMBER: builtins.int
    SHIELDS_FIELD_NUMBER: builtins.int
    ENERGY_FIELD_NUMBER: builtins.int
    TRANSPORT_SLOTS_TAKEN_FIELD_NUMBER: builtins.int
    BUILD_PROGRESS_FIELD_NUMBER: builtins.int
    ADD_ON_FIELD_NUMBER: builtins.int
    MAX_HEALTH_FIELD_NUMBER: builtins.int
    MAX_SHIELDS_FIELD_NUMBER: builtins.int
    MAX_ENERGY_FIELD_NUMBER: builtins.int
    unit_type: builtins.int
    player_relative: builtins.int
    health: builtins.int
    shields: builtins.int
    energy: builtins.int
    transport_slots_taken: builtins.int
    build_progress: builtins.float
    """Range: [0.0, 1.0]"""
    @property
    def add_on(self) -> global___UnitInfo: ...
    max_health: builtins.int
    max_shields: builtins.int
    max_energy: builtins.int
    def __init__(
        self,
        *,
        unit_type: builtins.int | None = ...,
        player_relative: builtins.int | None = ...,
        health: builtins.int | None = ...,
        shields: builtins.int | None = ...,
        energy: builtins.int | None = ...,
        transport_slots_taken: builtins.int | None = ...,
        build_progress: builtins.float | None = ...,
        add_on: global___UnitInfo | None = ...,
        max_health: builtins.int | None = ...,
        max_shields: builtins.int | None = ...,
        max_energy: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["add_on", b"add_on", "build_progress", b"build_progress", "energy", b"energy", "health", b"health", "max_energy", b"max_energy", "max_health", b"max_health", "max_shields", b"max_shields", "player_relative", b"player_relative", "shields", b"shields", "transport_slots_taken", b"transport_slots_taken", "unit_type", b"unit_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["add_on", b"add_on", "build_progress", b"build_progress", "energy", b"energy", "health", b"health", "max_energy", b"max_energy", "max_health", b"max_health", "max_shields", b"max_shields", "player_relative", b"player_relative", "shields", b"shields", "transport_slots_taken", b"transport_slots_taken", "unit_type", b"unit_type"]) -> None: ...

global___UnitInfo = UnitInfo

@typing_extensions.final
class SinglePanel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_FIELD_NUMBER: builtins.int
    ATTACK_UPGRADE_LEVEL_FIELD_NUMBER: builtins.int
    ARMOR_UPGRADE_LEVEL_FIELD_NUMBER: builtins.int
    SHIELD_UPGRADE_LEVEL_FIELD_NUMBER: builtins.int
    BUFFS_FIELD_NUMBER: builtins.int
    @property
    def unit(self) -> global___UnitInfo: ...
    attack_upgrade_level: builtins.int
    armor_upgrade_level: builtins.int
    shield_upgrade_level: builtins.int
    @property
    def buffs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        unit: global___UnitInfo | None = ...,
        attack_upgrade_level: builtins.int | None = ...,
        armor_upgrade_level: builtins.int | None = ...,
        shield_upgrade_level: builtins.int | None = ...,
        buffs: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["armor_upgrade_level", b"armor_upgrade_level", "attack_upgrade_level", b"attack_upgrade_level", "shield_upgrade_level", b"shield_upgrade_level", "unit", b"unit"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["armor_upgrade_level", b"armor_upgrade_level", "attack_upgrade_level", b"attack_upgrade_level", "buffs", b"buffs", "shield_upgrade_level", b"shield_upgrade_level", "unit", b"unit"]) -> None: ...

global___SinglePanel = SinglePanel

@typing_extensions.final
class MultiPanel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNITS_FIELD_NUMBER: builtins.int
    @property
    def units(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___UnitInfo]: ...
    def __init__(
        self,
        *,
        units: collections.abc.Iterable[global___UnitInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["units", b"units"]) -> None: ...

global___MultiPanel = MultiPanel

@typing_extensions.final
class CargoPanel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_FIELD_NUMBER: builtins.int
    PASSENGERS_FIELD_NUMBER: builtins.int
    SLOTS_AVAILABLE_FIELD_NUMBER: builtins.int
    @property
    def unit(self) -> global___UnitInfo: ...
    @property
    def passengers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___UnitInfo]: ...
    slots_available: builtins.int
    """TODO: Change to cargo size"""
    def __init__(
        self,
        *,
        unit: global___UnitInfo | None = ...,
        passengers: collections.abc.Iterable[global___UnitInfo] | None = ...,
        slots_available: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["slots_available", b"slots_available", "unit", b"unit"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["passengers", b"passengers", "slots_available", b"slots_available", "unit", b"unit"]) -> None: ...

global___CargoPanel = CargoPanel

@typing_extensions.final
class BuildItem(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITY_ID_FIELD_NUMBER: builtins.int
    BUILD_PROGRESS_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    build_progress: builtins.float
    """Range: [0.0, 1.0]"""
    def __init__(
        self,
        *,
        ability_id: builtins.int | None = ...,
        build_progress: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "build_progress", b"build_progress"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "build_progress", b"build_progress"]) -> None: ...

global___BuildItem = BuildItem

@typing_extensions.final
class ProductionPanel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_FIELD_NUMBER: builtins.int
    BUILD_QUEUE_FIELD_NUMBER: builtins.int
    PRODUCTION_QUEUE_FIELD_NUMBER: builtins.int
    @property
    def unit(self) -> global___UnitInfo: ...
    @property
    def build_queue(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___UnitInfo]:
        """build_queue ONLY gives information about units that are being produced.
        Use production_queue instead to see both units being trained as well as research in the queue.
        """
    @property
    def production_queue(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BuildItem]: ...
    def __init__(
        self,
        *,
        unit: global___UnitInfo | None = ...,
        build_queue: collections.abc.Iterable[global___UnitInfo] | None = ...,
        production_queue: collections.abc.Iterable[global___BuildItem] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["unit", b"unit"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_queue", b"build_queue", "production_queue", b"production_queue", "unit", b"unit"]) -> None: ...

global___ProductionPanel = ProductionPanel

@typing_extensions.final
class ActionUI(google.protobuf.message.Message):
    """
    Action
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTROL_GROUP_FIELD_NUMBER: builtins.int
    SELECT_ARMY_FIELD_NUMBER: builtins.int
    SELECT_WARP_GATES_FIELD_NUMBER: builtins.int
    SELECT_LARVA_FIELD_NUMBER: builtins.int
    SELECT_IDLE_WORKER_FIELD_NUMBER: builtins.int
    MULTI_PANEL_FIELD_NUMBER: builtins.int
    CARGO_PANEL_FIELD_NUMBER: builtins.int
    PRODUCTION_PANEL_FIELD_NUMBER: builtins.int
    TOGGLE_AUTOCAST_FIELD_NUMBER: builtins.int
    @property
    def control_group(self) -> global___ActionControlGroup: ...
    @property
    def select_army(self) -> global___ActionSelectArmy: ...
    @property
    def select_warp_gates(self) -> global___ActionSelectWarpGates: ...
    @property
    def select_larva(self) -> global___ActionSelectLarva: ...
    @property
    def select_idle_worker(self) -> global___ActionSelectIdleWorker: ...
    @property
    def multi_panel(self) -> global___ActionMultiPanel: ...
    @property
    def cargo_panel(self) -> global___ActionCargoPanelUnload: ...
    @property
    def production_panel(self) -> global___ActionProductionPanelRemoveFromQueue: ...
    @property
    def toggle_autocast(self) -> global___ActionToggleAutocast: ...
    def __init__(
        self,
        *,
        control_group: global___ActionControlGroup | None = ...,
        select_army: global___ActionSelectArmy | None = ...,
        select_warp_gates: global___ActionSelectWarpGates | None = ...,
        select_larva: global___ActionSelectLarva | None = ...,
        select_idle_worker: global___ActionSelectIdleWorker | None = ...,
        multi_panel: global___ActionMultiPanel | None = ...,
        cargo_panel: global___ActionCargoPanelUnload | None = ...,
        production_panel: global___ActionProductionPanelRemoveFromQueue | None = ...,
        toggle_autocast: global___ActionToggleAutocast | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["action", b"action", "cargo_panel", b"cargo_panel", "control_group", b"control_group", "multi_panel", b"multi_panel", "production_panel", b"production_panel", "select_army", b"select_army", "select_idle_worker", b"select_idle_worker", "select_larva", b"select_larva", "select_warp_gates", b"select_warp_gates", "toggle_autocast", b"toggle_autocast"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["action", b"action", "cargo_panel", b"cargo_panel", "control_group", b"control_group", "multi_panel", b"multi_panel", "production_panel", b"production_panel", "select_army", b"select_army", "select_idle_worker", b"select_idle_worker", "select_larva", b"select_larva", "select_warp_gates", b"select_warp_gates", "toggle_autocast", b"toggle_autocast"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["action", b"action"]) -> typing_extensions.Literal["control_group", "select_army", "select_warp_gates", "select_larva", "select_idle_worker", "multi_panel", "cargo_panel", "production_panel", "toggle_autocast"] | None: ...

global___ActionUI = ActionUI

@typing_extensions.final
class ActionControlGroup(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ControlGroupAction:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ControlGroupActionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ActionControlGroup._ControlGroupAction.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        Recall: ActionControlGroup._ControlGroupAction.ValueType  # 1
        """Equivalent to number hotkey. Replaces current selection with control group."""
        Set: ActionControlGroup._ControlGroupAction.ValueType  # 2
        """Equivalent to Control + number hotkey. Sets control group to current selection."""
        Append: ActionControlGroup._ControlGroupAction.ValueType  # 3
        """Equivalent to Shift + number hotkey. Adds current selection into control group."""
        SetAndSteal: ActionControlGroup._ControlGroupAction.ValueType  # 4
        """Equivalent to Control + Alt + number hotkey. Sets control group to current selection. Units are removed from other control groups."""
        AppendAndSteal: ActionControlGroup._ControlGroupAction.ValueType  # 5
        """Equivalent to Shift + Alt + number hotkey. Adds current selection into control group. Units are removed from other control groups."""

    class ControlGroupAction(_ControlGroupAction, metaclass=_ControlGroupActionEnumTypeWrapper): ...
    Recall: ActionControlGroup.ControlGroupAction.ValueType  # 1
    """Equivalent to number hotkey. Replaces current selection with control group."""
    Set: ActionControlGroup.ControlGroupAction.ValueType  # 2
    """Equivalent to Control + number hotkey. Sets control group to current selection."""
    Append: ActionControlGroup.ControlGroupAction.ValueType  # 3
    """Equivalent to Shift + number hotkey. Adds current selection into control group."""
    SetAndSteal: ActionControlGroup.ControlGroupAction.ValueType  # 4
    """Equivalent to Control + Alt + number hotkey. Sets control group to current selection. Units are removed from other control groups."""
    AppendAndSteal: ActionControlGroup.ControlGroupAction.ValueType  # 5
    """Equivalent to Shift + Alt + number hotkey. Adds current selection into control group. Units are removed from other control groups."""

    ACTION_FIELD_NUMBER: builtins.int
    CONTROL_GROUP_INDEX_FIELD_NUMBER: builtins.int
    action: global___ActionControlGroup.ControlGroupAction.ValueType
    control_group_index: builtins.int
    def __init__(
        self,
        *,
        action: global___ActionControlGroup.ControlGroupAction.ValueType | None = ...,
        control_group_index: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["action", b"action", "control_group_index", b"control_group_index"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["action", b"action", "control_group_index", b"control_group_index"]) -> None: ...

global___ActionControlGroup = ActionControlGroup

@typing_extensions.final
class ActionSelectArmy(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SELECTION_ADD_FIELD_NUMBER: builtins.int
    selection_add: builtins.bool
    def __init__(
        self,
        *,
        selection_add: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["selection_add", b"selection_add"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["selection_add", b"selection_add"]) -> None: ...

global___ActionSelectArmy = ActionSelectArmy

@typing_extensions.final
class ActionSelectWarpGates(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SELECTION_ADD_FIELD_NUMBER: builtins.int
    selection_add: builtins.bool
    def __init__(
        self,
        *,
        selection_add: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["selection_add", b"selection_add"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["selection_add", b"selection_add"]) -> None: ...

global___ActionSelectWarpGates = ActionSelectWarpGates

@typing_extensions.final
class ActionSelectLarva(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ActionSelectLarva = ActionSelectLarva

@typing_extensions.final
class ActionSelectIdleWorker(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Type:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ActionSelectIdleWorker._Type.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        Set: ActionSelectIdleWorker._Type.ValueType  # 1
        """Equivalent to click with no modifiers. Replaces selection with single idle worker."""
        Add: ActionSelectIdleWorker._Type.ValueType  # 2
        """Equivalent to shift+click. Adds single idle worker to current selection."""
        All: ActionSelectIdleWorker._Type.ValueType  # 3
        """Equivalent to control+click. Selects all idle workers."""
        AddAll: ActionSelectIdleWorker._Type.ValueType  # 4
        """Equivalent to shift+control+click. Adds all idle workers to current selection."""

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    Set: ActionSelectIdleWorker.Type.ValueType  # 1
    """Equivalent to click with no modifiers. Replaces selection with single idle worker."""
    Add: ActionSelectIdleWorker.Type.ValueType  # 2
    """Equivalent to shift+click. Adds single idle worker to current selection."""
    All: ActionSelectIdleWorker.Type.ValueType  # 3
    """Equivalent to control+click. Selects all idle workers."""
    AddAll: ActionSelectIdleWorker.Type.ValueType  # 4
    """Equivalent to shift+control+click. Adds all idle workers to current selection."""

    TYPE_FIELD_NUMBER: builtins.int
    type: global___ActionSelectIdleWorker.Type.ValueType
    def __init__(
        self,
        *,
        type: global___ActionSelectIdleWorker.Type.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["type", b"type"]) -> None: ...

global___ActionSelectIdleWorker = ActionSelectIdleWorker

@typing_extensions.final
class ActionMultiPanel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Type:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ActionMultiPanel._Type.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SingleSelect: ActionMultiPanel._Type.ValueType  # 1
        """Click on icon"""
        DeselectUnit: ActionMultiPanel._Type.ValueType  # 2
        """Shift Click on icon"""
        SelectAllOfType: ActionMultiPanel._Type.ValueType  # 3
        """Control Click on icon."""
        DeselectAllOfType: ActionMultiPanel._Type.ValueType  # 4
        """Control+Shift Click on icon."""

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    SingleSelect: ActionMultiPanel.Type.ValueType  # 1
    """Click on icon"""
    DeselectUnit: ActionMultiPanel.Type.ValueType  # 2
    """Shift Click on icon"""
    SelectAllOfType: ActionMultiPanel.Type.ValueType  # 3
    """Control Click on icon."""
    DeselectAllOfType: ActionMultiPanel.Type.ValueType  # 4
    """Control+Shift Click on icon."""

    TYPE_FIELD_NUMBER: builtins.int
    UNIT_INDEX_FIELD_NUMBER: builtins.int
    type: global___ActionMultiPanel.Type.ValueType
    unit_index: builtins.int
    def __init__(
        self,
        *,
        type: global___ActionMultiPanel.Type.ValueType | None = ...,
        unit_index: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["type", b"type", "unit_index", b"unit_index"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["type", b"type", "unit_index", b"unit_index"]) -> None: ...

global___ActionMultiPanel = ActionMultiPanel

@typing_extensions.final
class ActionCargoPanelUnload(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_INDEX_FIELD_NUMBER: builtins.int
    unit_index: builtins.int
    def __init__(
        self,
        *,
        unit_index: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["unit_index", b"unit_index"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["unit_index", b"unit_index"]) -> None: ...

global___ActionCargoPanelUnload = ActionCargoPanelUnload

@typing_extensions.final
class ActionProductionPanelRemoveFromQueue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_INDEX_FIELD_NUMBER: builtins.int
    unit_index: builtins.int
    def __init__(
        self,
        *,
        unit_index: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["unit_index", b"unit_index"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["unit_index", b"unit_index"]) -> None: ...

global___ActionProductionPanelRemoveFromQueue = ActionProductionPanelRemoveFromQueue

@typing_extensions.final
class ActionToggleAutocast(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITY_ID_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    def __init__(
        self,
        *,
        ability_id: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id"]) -> None: ...

global___ActionToggleAutocast = ActionToggleAutocast



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str
logger: Incomplete

class Gcp(VaultApiBase):
    def configure(
        self,
        credentials: Incomplete | None = None,
        google_certs_endpoint="https://www.googleapis.com/oauth2/v3/certs",
        mount_point="gcp",
    ): ...
    def read_config(self, mount_point="gcp"): ...
    def delete_config(self, mount_point="gcp"): ...
    def create_role(
        self,
        name,
        role_type,
        project_id,
        ttl: Incomplete | None = None,
        max_ttl: Incomplete | None = None,
        period: Incomplete | None = None,
        policies: Incomplete | None = None,
        bound_service_accounts: Incomplete | None = None,
        max_jwt_exp: Incomplete | None = None,
        allow_gce_inference: Incomplete | None = None,
        bound_zones: Incomplete | None = None,
        bound_regions: Incomplete | None = None,
        bound_instance_groups: Incomplete | None = None,
        bound_labels: Incomplete | None = None,
        mount_point="gcp",
    ): ...
    def edit_service_accounts_on_iam_role(
        self, name, add: Incomplete | None = None, remove: Incomplete | None = None, mount_point="gcp"
    ): ...
    def edit_labels_on_gce_role(
        self, name, add: Incomplete | None = None, remove: Incomplete | None = None, mount_point="gcp"
    ): ...
    def read_role(self, name, mount_point="gcp"): ...
    def list_roles(self, mount_point="gcp"): ...
    def delete_role(self, role, mount_point="gcp"): ...
    def login(self, role, jwt, use_token: bool = True, mount_point="gcp"): ...



from influxdb_client import User, UserResponse, Users

class UsersApi:
    def __init__(self, influxdb_client) -> None: ...
    def me(self) -> User: ...
    def create_user(self, name: str) -> User: ...
    def update_user(self, user: User) -> UserResponse: ...
    def update_password(self, user: str | User | UserResponse, password: str) -> None: ...
    def delete_user(self, user: str | User | UserResponse) -> None: ...
    def find_users(self, **kwargs) -> Users: ...



from typing import Any

class Attribute:
    key: Any
    definition: Any
    values: Any
    raw_values: Any
    response: Any
    entry: Any
    cursor: Any
    other_names: Any
    def __init__(self, attr_def, entry, cursor) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def __getitem__(self, item): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    @property
    def value(self): ...

class OperationalAttribute(Attribute): ...

class WritableAttribute(Attribute):
    def __iadd__(self, other): ...
    def __isub__(self, other): ...
    def add(self, values) -> None: ...
    def set(self, values) -> None: ...
    def delete(self, values) -> None: ...
    def remove(self) -> None: ...
    def discard(self) -> None: ...
    @property
    def virtual(self): ...
    @property
    def changes(self): ...



from collections.abc import Iterable

m: list[list[float]]
m_inv: list[list[float]]
refX: float
refY: float
refZ: float
refU: float
refV: float
lab_e: float
lab_k: float

def husl_to_rgb(h: float, s: float, l: float) -> list[float]: ...
def husl_to_hex(h: float, s: float, l: float) -> str: ...
def rgb_to_husl(r: float, g: float, b: float) -> list[float]: ...
def hex_to_husl(hex: str) -> list[float]: ...
def huslp_to_rgb(h: float, s: float, l: float) -> list[float]: ...
def huslp_to_hex(h: float, s: float, l: float) -> str: ...
def rgb_to_huslp(r: float, g: float, b: float) -> list[float]: ...
def hex_to_huslp(hex: str) -> list[float]: ...
def lch_to_rgb(l: float, c: float, h: float) -> list[float]: ...
def rgb_to_lch(r: float, g: float, b: float) -> list[float]: ...
def max_chroma(L: float, H: float) -> float: ...
def max_chroma_pastel(L: float) -> float: ...
def dot_product(a: Iterable[float], b: Iterable[float]) -> float: ...
def f(t: float) -> float: ...
def f_inv(t: float) -> float: ...
def from_linear(c: float) -> float: ...
def to_linear(c: float) -> float: ...
def rgb_prepare(triple: Iterable[float]) -> list[int]: ...
def hex_to_rgb(hex: str) -> list[float]: ...
def rgb_to_hex(triple: Iterable[float]) -> str: ...
def xyz_to_rgb(triple: Iterable[float]) -> list[float]: ...
def rgb_to_xyz(triple: Iterable[float]) -> list[float]: ...
def xyz_to_luv(triple: Iterable[float]) -> list[float]: ...
def luv_to_xyz(triple: Iterable[float]) -> list[float]: ...
def luv_to_lch(triple: Iterable[float]) -> list[float]: ...
def lch_to_luv(triple: Iterable[float]) -> list[float]: ...
def husl_to_lch(triple: Iterable[float]) -> list[float]: ...
def lch_to_husl(triple: Iterable[float]) -> list[float]: ...
def huslp_to_lch(triple: Iterable[float]) -> list[float]: ...
def lch_to_huslp(triple: Iterable[float]) -> list[float]: ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class ViewsService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def get_dashboards_id_cells_id_view(self, dashboard_id, cell_id, **kwargs): ...
    def get_dashboards_id_cells_id_view_with_http_info(self, dashboard_id, cell_id, **kwargs): ...
    async def get_dashboards_id_cells_id_view_async(self, dashboard_id, cell_id, **kwargs): ...
    def patch_dashboards_id_cells_id_view(self, dashboard_id, cell_id, view, **kwargs): ...
    def patch_dashboards_id_cells_id_view_with_http_info(self, dashboard_id, cell_id, view, **kwargs): ...
    async def patch_dashboards_id_cells_id_view_async(self, dashboard_id, cell_id, view, **kwargs): ...



from .algeria import Algeria as Algeria
from .angola import Angola as Angola
from .benin import Benin as Benin
from .ivory_coast import IvoryCoast as IvoryCoast
from .kenya import Kenya as Kenya
from .madagascar import Madagascar as Madagascar
from .mozambique import Mozambique as Mozambique
from .nigeria import Nigeria as Nigeria
from .sao_tome import SaoTomeAndPrincipe as SaoTomeAndPrincipe
from .south_africa import SouthAfrica as SouthAfrica
from .tunisia import Tunisia as Tunisia



from networkx.algorithms.community.asyn_fluid import *
from networkx.algorithms.community.centrality import *
from networkx.algorithms.community.community_utils import *
from networkx.algorithms.community.kclique import *
from networkx.algorithms.community.kernighan_lin import *
from networkx.algorithms.community.label_propagation import *
from networkx.algorithms.community.louvain import *
from networkx.algorithms.community.lukes import *
from networkx.algorithms.community.modularity_max import *
from networkx.algorithms.community.quality import *



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Ssh(VaultApiBase):
    def create_or_update_key(self, name: str = "", key: str = "", mount_point: str = "ssh"): ...
    def delete_key(self, name: str = "", mount_point: str = "ssh"): ...
    def create_role(
        self,
        name: str = "",
        key: str = "",
        admin_user: str = "",
        default_user: str = "",
        cidr_list: str = "",
        exclude_cidr_list: str = "",
        port: int = 22,
        key_type: str = "",
        key_bits: int = 1024,
        install_script: str = "",
        allowed_users: str = "",
        allowed_users_template: str = "",
        allowed_domains: str = "",
        key_option_specs: str = "",
        ttl: str = "",
        max_ttl: str = "",
        allowed_critical_options: str = "",
        allowed_extensions: str = "",
        default_critical_options: Incomplete | None = None,
        default_extensions: Incomplete | None = None,
        allow_user_certificates: str = "",
        allow_host_certificates: bool = False,
        allow_bare_domains: bool = False,
        allow_subdomains: bool = False,
        allow_user_key_ids: bool = False,
        key_id_format: str = "",
        allowed_user_key_lengths: Incomplete | None = None,
        algorithm_signer: str = "",
        mount_point="ssh",
    ): ...
    def read_role(self, name: str = "", mount_point: str = "ssh"): ...
    def list_roles(self, mount_point: str = "ssh"): ...
    def delete_role(self, name: str = "", mount_point: str = "ssh"): ...
    def list_zeroaddress_roles(self, mount_point: str = "ssh"): ...
    def configure_zeroaddress_roles(self, roles: str = "", mount_point: str = "ssh"): ...
    def delete_zeroaddress_role(self, mount_point: str = "ssh"): ...
    def generate_ssh_credentials(self, name: str = "", username: str = "", ip: str = "", mount_point: str = "ssh"): ...
    def list_roles_by_ip(self, ip: str = "", mount_point: str = "ssh"): ...
    def verify_ssh_otp(self, otp, mount_point="ssh"): ...
    def submit_ca_information(
        self,
        private_key: str = "",
        public_key: str = "",
        generate_signing_key: bool = True,
        key_type: str = "ssh-rsa",
        key_bits: int = 0,
        mount_point: str = "ssh",
    ): ...
    def delete_ca_information(self, mount_point: str = "ssh"): ...
    def read_public_key(self, mount_point: str = "ssh"): ...
    def sign_ssh_key(
        self,
        name: str = "",
        public_key: str = "",
        ttl: str = "",
        valid_principals: str = "",
        cert_type: str = "user",
        key_id: str = "",
        critical_options: Incomplete | None = None,
        extensions: Incomplete | None = None,
        mount_point: str = "ssh",
    ): ...



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class KvV2(VaultApiBase):
    def configure(
        self,
        max_versions: int = 10,
        cas_required: Incomplete | None = None,
        delete_version_after: str = "0s",
        mount_point="secret",
    ): ...
    def read_configuration(self, mount_point="secret"): ...
    def read_secret(self, path, mount_point="secret", raise_on_deleted_version: Incomplete | None = None): ...
    def read_secret_version(
        self, path, version: Incomplete | None = None, mount_point="secret", raise_on_deleted_version: Incomplete | None = None
    ): ...
    def create_or_update_secret(self, path, secret, cas: Incomplete | None = None, mount_point="secret"): ...
    def patch(self, path, secret, mount_point="secret"): ...
    def delete_latest_version_of_secret(self, path, mount_point="secret"): ...
    def delete_secret_versions(self, path, versions, mount_point="secret"): ...
    def undelete_secret_versions(self, path, versions, mount_point="secret"): ...
    def destroy_secret_versions(self, path, versions, mount_point="secret"): ...
    def list_secrets(self, path, mount_point="secret"): ...
    def read_secret_metadata(self, path, mount_point="secret"): ...
    def update_metadata(
        self,
        path,
        max_versions: Incomplete | None = None,
        cas_required: Incomplete | None = None,
        delete_version_after: str = "0s",
        mount_point="secret",
        custom_metadata: Incomplete | None = None,
    ): ...
    def delete_metadata_and_all_versions(self, path, mount_point="secret"): ...



import datetime
import time
from _typeshed import Unused
from collections.abc import Callable, Mapping, Sequence
from decimal import Decimal
from typing import Any, TypeVar
from typing_extensions import TypeAlias

_EscaperMapping: TypeAlias = Mapping[type[object], Callable[..., str]] | None
_T = TypeVar("_T")

def escape_item(val: object, charset: object, mapping: _EscaperMapping = None) -> str: ...
def escape_dict(val: Mapping[str, object], charset: object, mapping: _EscaperMapping = None) -> dict[str, str]: ...
def escape_sequence(val: Sequence[object], charset: object, mapping: _EscaperMapping = None) -> str: ...
def escape_set(val: set[object], charset: object, mapping: _EscaperMapping = None) -> str: ...
def escape_bool(value: bool, mapping: _EscaperMapping = None) -> str: ...
def escape_int(value: int, mapping: _EscaperMapping = None) -> str: ...
def escape_float(value: float, mapping: _EscaperMapping = None) -> str: ...
def escape_string(value: str, mapping: _EscaperMapping = None) -> str: ...
def escape_bytes_prefixed(value: bytes, mapping: _EscaperMapping = None) -> str: ...
def escape_bytes(value: bytes, mapping: _EscaperMapping = None) -> str: ...
def escape_str(value: str, mapping: _EscaperMapping = None) -> str: ...
def escape_None(value: None, mapping: _EscaperMapping = None) -> str: ...
def escape_timedelta(obj: datetime.timedelta, mapping: _EscaperMapping = None) -> str: ...
def escape_time(obj: datetime.time, mapping: _EscaperMapping = None) -> str: ...
def escape_datetime(obj: datetime.datetime, mapping: _EscaperMapping = None) -> str: ...
def escape_date(obj: datetime.date, mapping: _EscaperMapping = None) -> str: ...
def escape_struct_time(obj: time.struct_time, mapping: _EscaperMapping = None) -> str: ...
def Decimal2Literal(o: Decimal, d: Unused) -> str: ...
def convert_datetime(obj: str | bytes) -> datetime.datetime | str: ...
def convert_timedelta(obj: str | bytes) -> datetime.timedelta | str: ...
def convert_time(obj: str | bytes) -> datetime.time | str: ...
def convert_date(obj: str | bytes) -> datetime.date | str: ...
def through(x: _T) -> _T: ...

convert_bit = through

encoders: dict[type[object], Callable[..., str]]
decoders: dict[int, Callable[[str | bytes], Any]]
conversions: dict[type[object] | int, Callable[..., Any]]
Thing2Literal = escape_str



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class DBRPsService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def delete_dbrpid(self, dbrp_id, **kwargs): ...
    def delete_dbrpid_with_http_info(self, dbrp_id, **kwargs): ...
    async def delete_dbrpid_async(self, dbrp_id, **kwargs): ...
    def get_dbr_ps(self, **kwargs): ...
    def get_dbr_ps_with_http_info(self, **kwargs): ...
    async def get_dbr_ps_async(self, **kwargs): ...
    def get_dbr_ps_id(self, dbrp_id, **kwargs): ...
    def get_dbr_ps_id_with_http_info(self, dbrp_id, **kwargs): ...
    async def get_dbr_ps_id_async(self, dbrp_id, **kwargs): ...
    def patch_dbrpid(self, dbrp_id, dbrp_update, **kwargs): ...
    def patch_dbrpid_with_http_info(self, dbrp_id, dbrp_update, **kwargs): ...
    async def patch_dbrpid_async(self, dbrp_id, dbrp_update, **kwargs): ...
    def post_dbrp(self, dbrp_create, **kwargs): ...
    def post_dbrp_with_http_info(self, dbrp_create, **kwargs): ...
    async def post_dbrp_async(self, dbrp_create, **kwargs): ...



from _typeshed import Incomplete
from typing import ClassVar

from openpyxl.descriptors.serialisable import Serialisable

class ExternalReference(Serialisable):
    tagname: ClassVar[str]
    id: Incomplete
    def __init__(self, id) -> None: ...



from typing import Any, ClassVar

from influxdb_client.domain.links import Links
from influxdb_client.domain.task import Task

class Tasks:
    openapi_types: ClassVar[dict[str, str]]
    attribute_map: ClassVar[dict[str, str]]

    tasks: list[Task]
    links: Links
    discriminator: None
    def __init__(self, links: Links | None = None, tasks: list[Task] | None = None) -> None: ...
    def to_dict(self) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import OrthodoxCalendar

class Georgia(OrthodoxCalendar):
    include_christmas: ClassVar[bool]
    include_christmas_eve: ClassVar[bool]
    include_new_years_day: ClassVar[bool]
    include_orthodox_christmas: ClassVar[bool]
    include_epiphany: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_easter_saturday: ClassVar[bool]
    include_easter_sunday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    FIXED_HOLIDAYS: Incomplete



from _typeshed import Incomplete

class MerchantAccountGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def create(self, params: Incomplete | None = None): ...
    def update(self, merchant_account_id, params: Incomplete | None = None): ...
    def find(self, merchant_account_id): ...
    def create_for_currency(self, params: Incomplete | None = None): ...
    def all(self): ...



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Github(VaultApiBase):
    def configure(
        self,
        organization,
        base_url: Incomplete | None = None,
        ttl: Incomplete | None = None,
        max_ttl: Incomplete | None = None,
        mount_point="github",
    ): ...
    def read_configuration(self, mount_point="github"): ...
    def map_team(self, team_name, policies: Incomplete | None = None, mount_point="github"): ...
    def read_team_mapping(self, team_name, mount_point="github"): ...
    def map_user(self, user_name, policies: Incomplete | None = None, mount_point="github"): ...
    def read_user_mapping(self, user_name, mount_point="github"): ...
    def login(self, token, use_token: bool = True, mount_point="github"): ...



from _typeshed import Incomplete
from typing import Any

class BaseLanguageDetector:
    languages: Any
    def __init__(self, languages) -> None: ...
    def iterate_applicable_languages(self, date_string, modify: bool = False, settings: Incomplete | None = None) -> None: ...

class AutoDetectLanguage(BaseLanguageDetector):
    language_pool: Any
    allow_redetection: Any
    def __init__(self, languages, allow_redetection: bool = False) -> None: ...
    languages: Any
    def iterate_applicable_languages(self, date_string, modify: bool = False, settings: Incomplete | None = None) -> None: ...

class ExactLanguages(BaseLanguageDetector):
    def __init__(self, languages) -> None: ...
    def iterate_applicable_languages(self, date_string, modify: bool = False, settings: Incomplete | None = None) -> None: ...



from _typeshed import Incomplete

from antlr4.atn.ATN import ATN as ATN
from antlr4.atn.ATNState import ATNState as ATNState, LoopEndState as LoopEndState, StarLoopEntryState as StarLoopEntryState
from antlr4.atn.ParserATNSimulator import ParserATNSimulator as ParserATNSimulator
from antlr4.atn.Transition import Transition as Transition
from antlr4.BufferedTokenStream import TokenStream as TokenStream
from antlr4.dfa.DFA import DFA as DFA
from antlr4.error.Errors import (
    FailedPredicateException as FailedPredicateException,
    RecognitionException as RecognitionException,
    UnsupportedOperationException as UnsupportedOperationException,
)
from antlr4.Lexer import Lexer as Lexer
from antlr4.Parser import Parser as Parser
from antlr4.ParserRuleContext import InterpreterRuleContext as InterpreterRuleContext, ParserRuleContext as ParserRuleContext
from antlr4.PredictionContext import PredictionContextCache as PredictionContextCache
from antlr4.Token import Token as Token

class ParserInterpreter(Parser):
    grammarFileName: Incomplete
    atn: Incomplete
    tokenNames: Incomplete
    ruleNames: Incomplete
    decisionToDFA: Incomplete
    sharedContextCache: Incomplete
    pushRecursionContextStates: Incomplete
    def __init__(
        self, grammarFileName: str, tokenNames: list[str], ruleNames: list[str], atn: ATN, input: TokenStream
    ) -> None: ...
    state: Incomplete
    def parse(self, startRuleIndex: int): ...
    def enterRecursionRule(self, localctx: ParserRuleContext, state: int, ruleIndex: int, precedence: int): ...
    def getATNState(self): ...
    def visitState(self, p: ATNState): ...
    def visitRuleStopState(self, p: ATNState): ...



import sys
from collections.abc import Callable, Mapping
from io import BytesIO
from logging import Logger
from socket import socket
from typing import Any
from typing_extensions import TypeAlias

from waitress import compat as compat, utilities as utilities

_Socket: TypeAlias = socket

socket_map: Mapping[int, socket]
map: Mapping[int, socket]

class ExitNow(Exception): ...

def read(obj: dispatcher) -> None: ...
def write(obj: dispatcher) -> None: ...
def readwrite(obj: dispatcher, flags: int) -> None: ...
def poll(timeout: float = 0.0, map: Mapping[int, socket] | None = None) -> None: ...
def poll2(timeout: float = 0.0, map: Mapping[int, socket] | None = None) -> None: ...

poll3 = poll2

def loop(
    timeout: float = 30.0, use_poll: bool = False, map: Mapping[int, socket] | None = None, count: int | None = None
) -> None: ...
def compact_traceback() -> tuple[tuple[str, str, str], BaseException, BaseException, str]: ...

class dispatcher:
    debug: bool
    connected: bool
    accepting: bool
    connecting: bool
    closing: bool
    addr: tuple[str, int] | None
    ignore_log_types: frozenset[Any]
    logger: Logger
    compact_traceback: Callable[[], tuple[tuple[str, str, str], BaseException, BaseException, str]]
    socket: _Socket | None
    def __init__(self, sock: _Socket | None = None, map: Mapping[int, _Socket] | None = None) -> None: ...
    def add_channel(self, map: Mapping[int, _Socket] | None = None) -> None: ...
    def del_channel(self, map: Mapping[int, _Socket] | None = None) -> None: ...
    family_and_type: tuple[int, int]
    def create_socket(self, family: int = ..., type: int = ...) -> None: ...
    def set_socket(self, sock: _Socket, map: Mapping[int, _Socket] | None = None) -> None: ...
    def set_reuse_addr(self) -> None: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def listen(self, num: int) -> None: ...
    def bind(self, addr: tuple[str, int]) -> None: ...
    def connect(self, address: tuple[str, int]) -> None: ...
    def accept(self) -> tuple[_Socket, tuple[str, int]] | None: ...
    def send(self, data: bytes, do_close: bool = True) -> int: ...
    def recv(self, buffer_size: int) -> bytes: ...
    def close(self) -> None: ...
    def log(self, message: str) -> None: ...
    def log_info(self, message: str, type: str = "info") -> None: ...
    def handle_read_event(self) -> None: ...
    def handle_connect_event(self) -> None: ...
    def handle_write_event(self) -> None: ...
    def handle_expt_event(self) -> None: ...
    def handle_error(self) -> None: ...
    def handle_expt(self) -> None: ...
    def handle_read(self) -> None: ...
    def handle_write(self) -> None: ...
    def handle_connect(self) -> None: ...
    def handle_accept(self) -> None: ...
    def handle_accepted(self, sock: _Socket, addr: Any) -> None: ...
    def handle_close(self) -> None: ...

class dispatcher_with_send(dispatcher):
    out_buffer: bytes
    def __init__(self, sock: socket | None = None, map: Mapping[int, socket] | None = None) -> None: ...
    def initiate_send(self) -> None: ...
    handle_write: Callable[[], None]
    def writable(self) -> bool: ...
    def send(self, data: bytes) -> None: ...  # type: ignore[override]

def close_all(map: Mapping[int, socket] | None = None, ignore_all: bool = False) -> None: ...

if sys.platform != "win32":
    class file_wrapper:
        fd: BytesIO
        def __init__(self, fd: BytesIO) -> None: ...
        def __del__(self) -> None: ...
        def recv(self, *args: Any) -> bytes: ...
        def send(self, *args: Any) -> bytes: ...
        def getsockopt(self, level: int, optname: int, buflen: bool | None = ...) -> int: ...
        read: Callable[..., bytes]
        write: Callable[..., bytes]
        def close(self) -> None: ...
        def fileno(self) -> BytesIO: ...

    class file_dispatcher(dispatcher):
        connected: bool
        def __init__(self, fd: BytesIO, map: Mapping[int, _Socket] | None = ...) -> None: ...
        socket: _Socket
        def set_file(self, fd: BytesIO) -> None: ...



from typing import Any

from ..cmd import Command

class install_data(Command):
    description: str
    user_options: Any
    boolean_options: Any
    install_dir: Any
    outfiles: Any
    root: Any
    force: int
    data_files: Any
    warn_dir: int
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def get_inputs(self): ...
    def get_outputs(self): ...



class MissingPivotFunction(UserWarning):
    @staticmethod
    def print_warning(query: str): ...

class CloudOnlyWarning(UserWarning):
    @staticmethod
    def print_warning(api_name: str, doc_url: str): ...



import sys
from _typeshed import StrOrBytesPath
from collections.abc import Generator, Iterable, Mapping
from contextlib import _GeneratorContextManager
from io import BufferedWriter
from typing import Any, AnyStr
from typing_extensions import TypeAlias

from . import compat, cookies, exceptions, structures
from .models import PreparedRequest, Request

_Uri: TypeAlias = str | bytes
OrderedDict = compat.OrderedDict
cookiejar_from_dict = cookies.cookiejar_from_dict
CaseInsensitiveDict = structures.CaseInsensitiveDict
InvalidURL = exceptions.InvalidURL

NETRC_FILES: tuple[str, str]
DEFAULT_CA_BUNDLE_PATH: Any
DEFAULT_PORTS: dict[str, int]
DEFAULT_ACCEPT_ENCODING: str

def dict_to_sequence(d): ...
def super_len(o): ...
def get_netrc_auth(url: _Uri, raise_errors: bool = False) -> tuple[str, str] | None: ...
def guess_filename(obj): ...
def extract_zipped_paths(path): ...
def atomic_open(filename: StrOrBytesPath) -> _GeneratorContextManager[BufferedWriter]: ...
def from_key_val_list(value): ...
def to_key_val_list(value): ...
def parse_list_header(value): ...
def parse_dict_header(value): ...
def unquote_header_value(value, is_filename: bool = False): ...
def dict_from_cookiejar(cj): ...
def add_dict_to_cookiejar(cj, cookie_dict): ...
def get_encodings_from_content(content): ...
def get_encoding_from_headers(headers: Mapping[str, str]) -> str | None: ...
def stream_decode_response_unicode(iterator, r): ...
def iter_slices(string: str, slice_length: int | None) -> Generator[str, None, None]: ...
def get_unicode_from_response(r): ...

UNRESERVED_SET: frozenset[str]

def unquote_unreserved(uri: str) -> str: ...
def requote_uri(uri: str) -> str: ...
def address_in_network(ip: str, net: str) -> bool: ...
def dotted_netmask(mask: int) -> str: ...
def is_ipv4_address(string_ip: str) -> bool: ...
def is_valid_cidr(string_network: str) -> bool: ...
def set_environ(env_name: str, value: None) -> _GeneratorContextManager[None]: ...
def should_bypass_proxies(url: _Uri, no_proxy: Iterable[str] | None) -> bool: ...
def get_environ_proxies(url: _Uri, no_proxy: Iterable[str] | None = None) -> dict[Any, Any]: ...
def select_proxy(url: _Uri, proxies: Mapping[Any, Any] | None): ...
def resolve_proxies(request: Request | PreparedRequest, proxies: Mapping[str, str] | None, trust_env: bool = True): ...
def default_user_agent(name: str = "python-requests") -> str: ...
def default_headers() -> CaseInsensitiveDict[str]: ...
def parse_header_links(value: str) -> list[dict[str, str]]: ...
def guess_json_utf(data): ...
def prepend_scheme_if_needed(url, new_scheme): ...
def get_auth_from_url(url: _Uri) -> tuple[str, str]: ...
def to_native_string(string, encoding="ascii"): ...
def urldefragauth(url: _Uri): ...
def rewind_body(prepared_request: PreparedRequest) -> None: ...
def check_header_validity(header: tuple[AnyStr, AnyStr]) -> None: ...

if sys.platform == "win32":
    def proxy_bypass_registry(host: str) -> bool: ...
    def proxy_bypass(host: str) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Callable, MutableMapping, Sequence
from types import ModuleType
from typing import Any, Generic, TypeVar, overload
from typing_extensions import Self

_T = TypeVar("_T")

WRAPPER_ASSIGNMENTS: tuple[str, ...]
WRAPPER_UPDATES: tuple[str, ...]

def update_wrapper(wrapper: _T, wrapped: object, assigned: Sequence[str] = ..., updated: Sequence[str] = ...) -> _T: ...
def copy_globals(
    source: ModuleType,
    globs: MutableMapping[str, Any],
    only_names: Incomplete | None = None,
    ignore_missing_names: bool = False,
    names_to_ignore: Sequence[str] = ...,
    dunder_names_to_keep: Sequence[str] = ...,
    cleanup_globs: bool = True,
) -> list[str]: ...
def import_c_accel(globs: MutableMapping[str, Any], cname: str) -> None: ...

class Lazy(Generic[_T]):
    data: _T
    def __init__(self, func: Callable[[Any], _T]) -> None: ...
    @overload
    def __get__(self, inst: None, class_: type[object]) -> Self: ...
    @overload
    def __get__(self, inst: object, class_: type[object]) -> _T: ...

class readproperty(Generic[_T]):
    func: Callable[[Any], _T]
    def __init__(self: readproperty[_T], func: Callable[[Any], _T]) -> None: ...
    @overload
    def __get__(self, inst: None, class_: type[object]) -> Self: ...
    @overload
    def __get__(self, inst: object, class_: type[object]) -> _T: ...

class LazyOnClass(Generic[_T]):
    @classmethod
    def lazy(cls, cls_dict: MutableMapping[str, Any], func: Callable[[Any], _T]) -> None: ...
    name: str
    func: Callable[[Any], _T]
    def __init__(self, func: Callable[[Any], _T], name: str | None = None) -> None: ...
    @overload
    def __get__(self, inst: None, class_: type[object]) -> Self: ...
    @overload
    def __get__(self, inst: object, class_: type[object]) -> _T: ...

def gmctime() -> str: ...
def prereleaser_middle(data: MutableMapping[str, Any]) -> None: ...
def postreleaser_before(data: MutableMapping[str, Any]) -> None: ...



from _typeshed import Incomplete
from typing import Any

from .base import BaseEndpoint as BaseEndpoint

log: Any

class RevocationEndpoint(BaseEndpoint):
    valid_token_types: Any
    valid_request_methods: Any
    request_validator: Any
    supported_token_types: Any
    enable_jsonp: Any
    def __init__(
        self, request_validator, supported_token_types: Incomplete | None = None, enable_jsonp: bool = False
    ) -> None: ...
    def create_revocation_response(
        self, uri, http_method: str = "POST", body: Incomplete | None = None, headers: Incomplete | None = None
    ): ...
    def validate_revocation_request(self, request) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, Literal

from ._imaging import _PixelAccessor
from .ImageFile import ImageFile

class BoxReader:
    fp: Incomplete
    has_length: Incomplete
    length: Incomplete
    remaining_in_box: int
    def __init__(self, fp, length: int = -1) -> None: ...
    def read_fields(self, field_format): ...
    def read_boxes(self): ...
    def has_next_box(self): ...
    def next_box_type(self): ...

class Jpeg2KImageFile(ImageFile):
    format: ClassVar[Literal["JPEG2000"]]
    format_description: ClassVar[str]
    reduce: Incomplete
    tile: Incomplete
    def load(self) -> _PixelAccessor: ...



from _typeshed import Incomplete
from typing import ClassVar

from .core import UnitedStates

class Wisconsin(UnitedStates):
    include_columbus_day: ClassVar[bool]
    include_federal_presidents_day: ClassVar[bool]
    include_christmas_eve: ClassVar[bool]
    FIXED_HOLIDAYS: Incomplete



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Author: kenton@google.com (Kenton Varda)

WARNING:  The plugin interface is currently EXPERIMENTAL and is subject to
  change.

protoc (aka the Protocol Compiler) can be extended via plugins.  A plugin is
just a program that reads a CodeGeneratorRequest from stdin and writes a
CodeGeneratorResponse to stdout.

Plugins written using C++ can use google/protobuf/compiler/plugin.h instead
of dealing with the raw protocol defined here.

A plugin executable needs only to be placed somewhere in the path.  The
plugin should be named "protoc-gen-$NAME", and will then be used when the
flag "--${NAME}_out" is passed to protoc.
"""
import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.descriptor_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Version(google.protobuf.message.Message):
    """The version number of protocol compiler."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAJOR_FIELD_NUMBER: builtins.int
    MINOR_FIELD_NUMBER: builtins.int
    PATCH_FIELD_NUMBER: builtins.int
    SUFFIX_FIELD_NUMBER: builtins.int
    major: builtins.int
    minor: builtins.int
    patch: builtins.int
    suffix: builtins.str
    """A suffix for alpha, beta or rc release, e.g., "alpha-1", "rc2". It should
    be empty for mainline stable releases.
    """
    def __init__(
        self,
        *,
        major: builtins.int | None = ...,
        minor: builtins.int | None = ...,
        patch: builtins.int | None = ...,
        suffix: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["major", b"major", "minor", b"minor", "patch", b"patch", "suffix", b"suffix"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["major", b"major", "minor", b"minor", "patch", b"patch", "suffix", b"suffix"]) -> None: ...

global___Version = Version

@typing_extensions.final
class CodeGeneratorRequest(google.protobuf.message.Message):
    """An encoded CodeGeneratorRequest is written to the plugin's stdin."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILE_TO_GENERATE_FIELD_NUMBER: builtins.int
    PARAMETER_FIELD_NUMBER: builtins.int
    PROTO_FILE_FIELD_NUMBER: builtins.int
    COMPILER_VERSION_FIELD_NUMBER: builtins.int
    @property
    def file_to_generate(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The .proto files that were explicitly listed on the command-line.  The
        code generator should generate code only for these files.  Each file's
        descriptor will be included in proto_file, below.
        """
    parameter: builtins.str
    """The generator parameter passed on the command-line."""
    @property
    def proto_file(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.descriptor_pb2.FileDescriptorProto]:
        """FileDescriptorProtos for all files in files_to_generate and everything
        they import.  The files will appear in topological order, so each file
        appears before any file that imports it.

        protoc guarantees that all proto_files will be written after
        the fields above, even though this is not technically guaranteed by the
        protobuf wire format.  This theoretically could allow a plugin to stream
        in the FileDescriptorProtos and handle them one by one rather than read
        the entire set into memory at once.  However, as of this writing, this
        is not similarly optimized on protoc's end -- it will store all fields in
        memory at once before sending them to the plugin.

        Type names of fields and extensions in the FileDescriptorProto are always
        fully qualified.
        """
    @property
    def compiler_version(self) -> global___Version:
        """The version number of protocol compiler."""
    def __init__(
        self,
        *,
        file_to_generate: collections.abc.Iterable[builtins.str] | None = ...,
        parameter: builtins.str | None = ...,
        proto_file: collections.abc.Iterable[google.protobuf.descriptor_pb2.FileDescriptorProto] | None = ...,
        compiler_version: global___Version | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["compiler_version", b"compiler_version", "parameter", b"parameter"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["compiler_version", b"compiler_version", "file_to_generate", b"file_to_generate", "parameter", b"parameter", "proto_file", b"proto_file"]) -> None: ...

global___CodeGeneratorRequest = CodeGeneratorRequest

@typing_extensions.final
class CodeGeneratorResponse(google.protobuf.message.Message):
    """The plugin writes an encoded CodeGeneratorResponse to stdout."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Feature:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _FeatureEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[CodeGeneratorResponse._Feature.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        FEATURE_NONE: CodeGeneratorResponse._Feature.ValueType  # 0
        FEATURE_PROTO3_OPTIONAL: CodeGeneratorResponse._Feature.ValueType  # 1

    class Feature(_Feature, metaclass=_FeatureEnumTypeWrapper):
        """Sync with code_generator.h."""

    FEATURE_NONE: CodeGeneratorResponse.Feature.ValueType  # 0
    FEATURE_PROTO3_OPTIONAL: CodeGeneratorResponse.Feature.ValueType  # 1

    @typing_extensions.final
    class File(google.protobuf.message.Message):
        """Represents a single generated file."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        INSERTION_POINT_FIELD_NUMBER: builtins.int
        CONTENT_FIELD_NUMBER: builtins.int
        GENERATED_CODE_INFO_FIELD_NUMBER: builtins.int
        name: builtins.str
        """The file name, relative to the output directory.  The name must not
        contain "." or ".." components and must be relative, not be absolute (so,
        the file cannot lie outside the output directory).  "/" must be used as
        the path separator, not "\\".

        If the name is omitted, the content will be appended to the previous
        file.  This allows the generator to break large files into small chunks,
        and allows the generated text to be streamed back to protoc so that large
        files need not reside completely in memory at one time.  Note that as of
        this writing protoc does not optimize for this -- it will read the entire
        CodeGeneratorResponse before writing files to disk.
        """
        insertion_point: builtins.str
        """If non-empty, indicates that the named file should already exist, and the
        content here is to be inserted into that file at a defined insertion
        point.  This feature allows a code generator to extend the output
        produced by another code generator.  The original generator may provide
        insertion points by placing special annotations in the file that look
        like:
          @@protoc_insertion_point(NAME)
        The annotation can have arbitrary text before and after it on the line,
        which allows it to be placed in a comment.  NAME should be replaced with
        an identifier naming the point -- this is what other generators will use
        as the insertion_point.  Code inserted at this point will be placed
        immediately above the line containing the insertion point (thus multiple
        insertions to the same point will come out in the order they were added).
        The double-@ is intended to make it unlikely that the generated code
        could contain things that look like insertion points by accident.

        For example, the C++ code generator places the following line in the
        .pb.h files that it generates:
          // @@protoc_insertion_point(namespace_scope)
        This line appears within the scope of the file's package namespace, but
        outside of any particular class.  Another plugin can then specify the
        insertion_point "namespace_scope" to generate additional classes or
        other declarations that should be placed in this scope.

        Note that if the line containing the insertion point begins with
        whitespace, the same whitespace will be added to every line of the
        inserted text.  This is useful for languages like Python, where
        indentation matters.  In these languages, the insertion point comment
        should be indented the same amount as any inserted code will need to be
        in order to work correctly in that context.

        The code generator that generates the initial file and the one which
        inserts into it must both run as part of a single invocation of protoc.
        Code generators are executed in the order in which they appear on the
        command line.

        If |insertion_point| is present, |name| must also be present.
        """
        content: builtins.str
        """The file contents."""
        @property
        def generated_code_info(self) -> google.protobuf.descriptor_pb2.GeneratedCodeInfo:
            """Information describing the file content being inserted. If an insertion
            point is used, this information will be appropriately offset and inserted
            into the code generation metadata for the generated files.
            """
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            insertion_point: builtins.str | None = ...,
            content: builtins.str | None = ...,
            generated_code_info: google.protobuf.descriptor_pb2.GeneratedCodeInfo | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["content", b"content", "generated_code_info", b"generated_code_info", "insertion_point", b"insertion_point", "name", b"name"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["content", b"content", "generated_code_info", b"generated_code_info", "insertion_point", b"insertion_point", "name", b"name"]) -> None: ...

    ERROR_FIELD_NUMBER: builtins.int
    SUPPORTED_FEATURES_FIELD_NUMBER: builtins.int
    FILE_FIELD_NUMBER: builtins.int
    error: builtins.str
    """Error message.  If non-empty, code generation failed.  The plugin process
    should exit with status code zero even if it reports an error in this way.

    This should be used to indicate errors in .proto files which prevent the
    code generator from generating correct code.  Errors which indicate a
    problem in protoc itself -- such as the input CodeGeneratorRequest being
    unparseable -- should be reported by writing a message to stderr and
    exiting with a non-zero status code.
    """
    supported_features: builtins.int
    """A bitmask of supported features that the code generator supports.
    This is a bitwise "or" of values from the Feature enum.
    """
    @property
    def file(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CodeGeneratorResponse.File]: ...
    def __init__(
        self,
        *,
        error: builtins.str | None = ...,
        supported_features: builtins.int | None = ...,
        file: collections.abc.Iterable[global___CodeGeneratorResponse.File] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["error", b"error", "supported_features", b"supported_features"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["error", b"error", "file", b"file", "supported_features", b"supported_features"]) -> None: ...

global___CodeGeneratorResponse = CodeGeneratorResponse



GIT_FEATURE_THREADS: int
GIT_FEATURE_HTTPS: int
GIT_FEATURE_SSH: int
GIT_FEATURE_NSEC: int
GIT_REPOSITORY_INIT_BARE: int
GIT_REPOSITORY_INIT_NO_REINIT: int
GIT_REPOSITORY_INIT_NO_DOTGIT_DIR: int
GIT_REPOSITORY_INIT_MKDIR: int
GIT_REPOSITORY_INIT_MKPATH: int
GIT_REPOSITORY_INIT_EXTERNAL_TEMPLATE: int
GIT_REPOSITORY_INIT_RELATIVE_GITLINK: int
GIT_REPOSITORY_INIT_SHARED_UMASK: int
GIT_REPOSITORY_INIT_SHARED_GROUP: int
GIT_REPOSITORY_INIT_SHARED_ALL: int
GIT_REPOSITORY_OPEN_NO_SEARCH: int
GIT_REPOSITORY_OPEN_CROSS_FS: int
GIT_REPOSITORY_OPEN_BARE: int
GIT_REPOSITORY_OPEN_NO_DOTGIT: int
GIT_REPOSITORY_OPEN_FROM_ENV: int
GIT_REPOSITORY_STATE_NONE: int
GIT_REPOSITORY_STATE_MERGE: int
GIT_REPOSITORY_STATE_REVERT: int
GIT_REPOSITORY_STATE_REVERT_SEQUENCE: int
GIT_REPOSITORY_STATE_CHERRYPICK: int
GIT_REPOSITORY_STATE_CHERRYPICK_SEQUENCE: int
GIT_REPOSITORY_STATE_BISECT: int
GIT_REPOSITORY_STATE_REBASE: int
GIT_REPOSITORY_STATE_REBASE_INTERACTIVE: int
GIT_REPOSITORY_STATE_REBASE_MERGE: int
GIT_REPOSITORY_STATE_APPLY_MAILBOX: int
GIT_REPOSITORY_STATE_APPLY_MAILBOX_OR_REBASE: int
GIT_ATTR_CHECK_FILE_THEN_INDEX: int
GIT_ATTR_CHECK_INDEX_THEN_FILE: int
GIT_ATTR_CHECK_INDEX_ONLY: int
GIT_ATTR_CHECK_NO_SYSTEM: int
GIT_ATTR_CHECK_INCLUDE_HEAD: int
GIT_ATTR_CHECK_INCLUDE_COMMIT: int
GIT_FETCH_PRUNE_UNSPECIFIED: int
GIT_FETCH_PRUNE: int
GIT_FETCH_NO_PRUNE: int
GIT_CHECKOUT_NOTIFY_NONE: int
GIT_CHECKOUT_NOTIFY_CONFLICT: int
GIT_CHECKOUT_NOTIFY_DIRTY: int
GIT_CHECKOUT_NOTIFY_UPDATED: int
GIT_CHECKOUT_NOTIFY_UNTRACKED: int
GIT_CHECKOUT_NOTIFY_IGNORED: int
GIT_CHECKOUT_NOTIFY_ALL: int
GIT_STASH_APPLY_PROGRESS_NONE: int
GIT_STASH_APPLY_PROGRESS_LOADING_STASH: int
GIT_STASH_APPLY_PROGRESS_ANALYZE_INDEX: int
GIT_STASH_APPLY_PROGRESS_ANALYZE_MODIFIED: int
GIT_STASH_APPLY_PROGRESS_ANALYZE_UNTRACKED: int
GIT_STASH_APPLY_PROGRESS_CHECKOUT_UNTRACKED: int
GIT_STASH_APPLY_PROGRESS_CHECKOUT_MODIFIED: int
GIT_STASH_APPLY_PROGRESS_DONE: int
GIT_CREDENTIAL_USERPASS_PLAINTEXT: int
GIT_CREDENTIAL_SSH_KEY: int
GIT_CREDENTIAL_SSH_CUSTOM: int
GIT_CREDENTIAL_DEFAULT: int
GIT_CREDENTIAL_SSH_INTERACTIVE: int
GIT_CREDENTIAL_USERNAME: int
GIT_CREDENTIAL_SSH_MEMORY: int



from _typeshed import Incomplete

from win32.lib.pywintypes import error as error

def GetExpandedName(Source, /) -> str: ...
def Close(handle, /) -> None: ...
def Copy(hSrc, hDest, /): ...
def Init(handle, /) -> None: ...
def OpenFile(fileName: str, action, /) -> tuple[Incomplete, Incomplete]: ...



from _typeshed import Incomplete
from collections.abc import Sequence
from typing import NamedTuple
from typing_extensions import Self

from .enums import Align, WrapMode
from .image_datastructures import RasterImageInfo, VectorImageInfo, _AlignLiteral

class Extents(NamedTuple):
    left: float
    right: float

class TextRegionMixin:
    def __init__(self, *args, **kwargs) -> None: ...
    def register_text_region(self, region) -> None: ...
    def is_current_text_region(self, region): ...
    def clear_text_region(self) -> None: ...

class LineWrapper(NamedTuple):
    line: Sequence[Incomplete]
    paragraph: Paragraph
    first_line: bool = False
    last_line: bool = False

class Paragraph:
    pdf: Incomplete
    text_align: Incomplete
    line_height: Incomplete
    top_margin: Incomplete
    bottom_margin: Incomplete
    skip_leading_spaces: Incomplete
    wrapmode: Incomplete

    def __init__(
        self,
        region,
        text_align: Incomplete | None = None,
        line_height: Incomplete | None = None,
        top_margin: float = 0,
        bottom_margin: float = 0,
        skip_leading_spaces: bool = False,
        wrapmode: WrapMode | None = None,
    ) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_value, traceback) -> None: ...
    def write(self, text: str, link: Incomplete | None = None): ...
    def ln(self, h: float | None = None) -> None: ...
    def build_lines(self, print_sh: bool) -> list[LineWrapper]: ...

class ImageParagraph:
    region: Incomplete
    name: Incomplete
    align: Align | None
    width: float | None
    height: float | None
    fill_width: bool
    keep_aspect_ratio: bool
    top_margin: float
    bottom_margin: float
    link: Incomplete | None
    title: Incomplete | None
    alt_text: Incomplete | None
    img: Incomplete | None
    info: Incomplete | None

    def __init__(
        self,
        region,
        name,
        align: Align | _AlignLiteral | None = None,
        width: float | None = None,
        height: float | None = None,
        fill_width: bool = False,
        keep_aspect_ratio: bool = False,
        top_margin: float = 0,
        bottom_margin: float = 0,
        link: Incomplete | None = None,
        title: Incomplete | None = None,
        alt_text: Incomplete | None = None,
    ) -> None: ...
    def build_line(self) -> Self: ...
    def render(self, col_left: float, col_width: float, max_height: float) -> VectorImageInfo | RasterImageInfo: ...

class ParagraphCollectorMixin:
    pdf: Incomplete
    text_align: Align
    line_height: Incomplete
    print_sh: Incomplete
    wrapmode: Incomplete
    skip_leading_spaces: Incomplete
    def __init__(
        self,
        pdf,
        *args,
        text: str | None = None,
        text_align: Align | _AlignLiteral = "LEFT",
        line_height: float = 1.0,
        print_sh: bool = False,
        skip_leading_spaces: bool = False,
        wrapmode: WrapMode | None = None,
        img: Incomplete | None = None,
        img_fill_width: bool = False,
        **kwargs,
    ) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_value, traceback) -> None: ...
    def write(self, text: str, link: Incomplete | None = None): ...
    def ln(self, h: float | None = None) -> None: ...
    def paragraph(
        self,
        text_align: Incomplete | None = None,
        line_height: Incomplete | None = None,
        skip_leading_spaces: bool = False,
        top_margin: int = 0,
        bottom_margin: int = 0,
        wrapmode: WrapMode | None = None,
    ): ...
    def end_paragraph(self) -> None: ...
    def image(
        self,
        name,
        align: Align | _AlignLiteral | None = None,
        width: float | None = None,
        height: float | None = None,
        fill_width: bool = False,
        keep_aspect_ratio: bool = False,
        top_margin: float = 0,
        bottom_margin: float = 0,
        link: Incomplete | None = None,
        title: Incomplete | None = None,
        alt_text: Incomplete | None = None,
    ) -> None: ...

class TextRegion(ParagraphCollectorMixin):
    def current_x_extents(self, y, height) -> None: ...
    def collect_lines(self): ...
    def render(self) -> None: ...
    def get_width(self, height): ...

class TextColumnarMixin:
    l_margin: Incomplete
    r_margin: Incomplete
    def __init__(self, pdf, *args, l_margin: Incomplete | None = None, r_margin: Incomplete | None = None, **kwargs) -> None: ...

class TextColumns(TextRegion, TextColumnarMixin):
    balance: Incomplete
    def __init__(self, pdf, *args, ncols: int = 1, gutter: float = 10, balance: bool = False, **kwargs) -> None: ...
    def __enter__(self) -> Self: ...
    def new_column(self) -> None: ...
    def render(self) -> None: ...
    def current_x_extents(self, y, height): ...



from matplotlib.colors import ListedColormap

# generated
rocket: ListedColormap
mako: ListedColormap
icefire: ListedColormap
vlag: ListedColormap
flare: ListedColormap
crest: ListedColormap
rocket_r: ListedColormap
mako_r: ListedColormap
icefire_r: ListedColormap
vlag_r: ListedColormap
flare_r: ListedColormap
crest_r: ListedColormap



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol Buffers - Google's data interchange format
Copyright 2008 Google Inc.  All rights reserved.
https://developers.google.com/protocol-buffers/

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following disclaimer
in the documentation and/or other materials provided with the
distribution.
    * Neither the name of Google Inc. nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import builtins
import collections.abc
import typing as typing_extensions

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.well_known_types
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class FieldMask(google.protobuf.message.Message, google.protobuf.internal.well_known_types.FieldMask):
    """`FieldMask` represents a set of symbolic field paths, for example:

        paths: "f.a"
        paths: "f.b.d"

    Here `f` represents a field in some root message, `a` and `b`
    fields in the message found in `f`, and `d` a field found in the
    message in `f.b`.

    Field masks are used to specify a subset of fields that should be
    returned by a get operation or modified by an update operation.
    Field masks also have a custom JSON encoding (see below).

    # Field Masks in Projections

    When used in the context of a projection, a response message or
    sub-message is filtered by the API to only contain those fields as
    specified in the mask. For example, if the mask in the previous
    example is applied to a response message as follows:

        f {
          a : 22
          b {
            d : 1
            x : 2
          }
          y : 13
        }
        z: 8

    The result will not contain specific values for fields x,y and z
    (their value will be set to the default, and omitted in proto text
    output):


        f {
          a : 22
          b {
            d : 1
          }
        }

    A repeated field is not allowed except at the last position of a
    paths string.

    If a FieldMask object is not present in a get operation, the
    operation applies to all fields (as if a FieldMask of all fields
    had been specified).

    Note that a field mask does not necessarily apply to the
    top-level response message. In case of a REST get operation, the
    field mask applies directly to the response, but in case of a REST
    list operation, the mask instead applies to each individual message
    in the returned resource list. In case of a REST custom method,
    other definitions may be used. Where the mask applies will be
    clearly documented together with its declaration in the API.  In
    any case, the effect on the returned resource/resources is required
    behavior for APIs.

    # Field Masks in Update Operations

    A field mask in update operations specifies which fields of the
    targeted resource are going to be updated. The API is required
    to only change the values of the fields as specified in the mask
    and leave the others untouched. If a resource is passed in to
    describe the updated values, the API ignores the values of all
    fields not covered by the mask.

    If a repeated field is specified for an update operation, new values will
    be appended to the existing repeated field in the target resource. Note that
    a repeated field is only allowed in the last position of a `paths` string.

    If a sub-message is specified in the last position of the field mask for an
    update operation, then new value will be merged into the existing sub-message
    in the target resource.

    For example, given the target message:

        f {
          b {
            d: 1
            x: 2
          }
          c: [1]
        }

    And an update message:

        f {
          b {
            d: 10
          }
          c: [2]
        }

    then if the field mask is:

     paths: ["f.b", "f.c"]

    then the result will be:

        f {
          b {
            d: 10
            x: 2
          }
          c: [1, 2]
        }

    An implementation may provide options to override this default behavior for
    repeated and message fields.

    In order to reset a field's value to the default, the field must
    be in the mask and set to the default value in the provided resource.
    Hence, in order to reset all fields of a resource, provide a default
    instance of the resource and set all fields in the mask, or do
    not provide a mask as described below.

    If a field mask is not present on update, the operation applies to
    all fields (as if a field mask of all fields has been specified).
    Note that in the presence of schema evolution, this may mean that
    fields the client does not know and has therefore not filled into
    the request will be reset to their default. If this is unwanted
    behavior, a specific service may require a client to always specify
    a field mask, producing an error if not.

    As with get operations, the location of the resource which
    describes the updated values in the request message depends on the
    operation kind. In any case, the effect of the field mask is
    required to be honored by the API.

    ## Considerations for HTTP REST

    The HTTP kind of an update operation which uses a field mask must
    be set to PATCH instead of PUT in order to satisfy HTTP semantics
    (PUT must only be used for full updates).

    # JSON Encoding of Field Masks

    In JSON, a field mask is encoded as a single string where paths are
    separated by a comma. Fields name in each path are converted
    to/from lower-camel naming conventions.

    As an example, consider the following message declarations:

        message Profile {
          User user = 1;
          Photo photo = 2;
        }
        message User {
          string display_name = 1;
          string address = 2;
        }

    In proto a field mask for `Profile` may look as such:

        mask {
          paths: "user.display_name"
          paths: "photo"
        }

    In JSON, the same mask is represented as below:

        {
          mask: "user.displayName,photo"
        }

    # Field Masks and Oneof Fields

    Field masks treat fields in oneofs just as regular fields. Consider the
    following message:

        message SampleMessage {
          oneof test_oneof {
            string name = 4;
            SubMessage sub_message = 9;
          }
        }

    The field mask can be:

        mask {
          paths: "name"
        }

    Or:

        mask {
          paths: "sub_message"
        }

    Note that oneof type names ("test_oneof" in this case) cannot be used in
    paths.

    ## Field Mask Verification

    The implementation of any API method which has a FieldMask type field in the
    request should verify the included field paths, and return an
    `INVALID_ARGUMENT` error if any path is unmappable.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATHS_FIELD_NUMBER: builtins.int
    @property
    def paths(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The set of field mask paths."""
    def __init__(
        self,
        *,
        paths: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["paths", b"paths"]) -> None: ...

global___FieldMask = FieldMask



from _typeshed import Incomplete

logger: Incomplete

class DataframeSerializer:
    data_frame: Incomplete
    f: Incomplete
    field_indexes: Incomplete
    first_field_maybe_null: Incomplete
    chunk_size: Incomplete
    def __init__(self, data_frame, point_settings, precision="ns", chunk_size: int | None = None, **kwargs) -> None: ...
    def serialize(self, chunk_idx: int | None = None): ...
    def number_of_chunks(self): ...

def data_frame_to_list_of_points(data_frame, point_settings, precision="ns", **kwargs): ...



from .__version__ import (
    __author__ as __author__,
    __author_email__ as __author_email__,
    __copyright__ as __copyright__,
    __description__ as __description__,
    __license__ as __license__,
    __title__ as __title__,
    __url__ as __url__,
    __version__ as __version__,
)
from .slugify import *
from .special import *



from _typeshed import Incomplete
from typing import Any

from .base import BaseEndpoint as BaseEndpoint

log: Any

class SignatureOnlyEndpoint(BaseEndpoint):
    def validate_request(
        self, uri, http_method: str = "GET", body: Incomplete | None = None, headers: Incomplete | None = None
    ): ...



import random
from _typeshed import Incomplete
from types import ModuleType
from typing_extensions import TypeAlias

import numpy

_RandomNumberGenerator: TypeAlias = (
    ModuleType | random.Random | numpy.random.RandomState | numpy.random.Generator | PythonRandomInterface
)
_RandomState: TypeAlias = int | _RandomNumberGenerator | None

def flatten(obj, result: Incomplete | None = None): ...
def make_list_of_ints(sequence): ...
def dict_to_numpy_array(d, mapping: Incomplete | None = None): ...
def arbitrary_element(iterable): ...
def pairwise(iterable, cyclic: bool = False): ...
def groups(many_to_one): ...
def create_random_state(random_state: Incomplete | None = None): ...

class PythonRandomInterface:
    def __init__(self, rng: Incomplete | None = None) -> None: ...
    def random(self): ...
    def uniform(self, a, b): ...
    def randrange(self, a, b: Incomplete | None = None): ...
    def choice(self, seq): ...
    def gauss(self, mu, sigma): ...
    def shuffle(self, seq): ...
    def sample(self, seq, k): ...
    def randint(self, a, b): ...
    def expovariate(self, scale): ...
    def paretovariate(self, shape): ...

def create_py_random_state(random_state: _RandomState = None): ...
def nodes_equal(nodes1, nodes2): ...
def edges_equal(edges1, edges2): ...
def graphs_equal(graph1, graph2): ...



from _typeshed import Incomplete
from collections.abc import Callable

from urllib3 import Retry

logger: Incomplete

class WritesRetry(Retry):
    jitter_interval: Incomplete
    total: Incomplete
    retry_interval: Incomplete
    max_retry_delay: Incomplete
    max_retry_time: Incomplete
    exponential_base: Incomplete
    retry_timeout: Incomplete
    retry_callback: Incomplete
    def __init__(
        self,
        jitter_interval: int = 0,
        max_retry_delay: int = 125,
        exponential_base: int = 2,
        max_retry_time: int = 180,
        total: int = 5,
        retry_interval: int = 5,
        retry_callback: Callable[[Exception], int] | None = None,
        **kw,
    ) -> None: ...
    def new(self, **kw): ...
    def is_retry(self, method, status_code, has_retry_after: bool = False): ...
    def get_backoff_time(self): ...
    def get_retry_after(self, response): ...
    def increment(
        self,
        method: Incomplete | None = None,
        url: Incomplete | None = None,
        response: Incomplete | None = None,
        error: Incomplete | None = None,
        _pool: Incomplete | None = None,
        _stacktrace: Incomplete | None = None,
    ): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def LCF_graph(n, shift_list, repeats, create_using: Incomplete | None = None): ...
@_dispatch
def bull_graph(create_using: Incomplete | None = None): ...
@_dispatch
def chvatal_graph(create_using: Incomplete | None = None): ...
@_dispatch
def cubical_graph(create_using: Incomplete | None = None): ...
@_dispatch
def desargues_graph(create_using: Incomplete | None = None): ...
@_dispatch
def diamond_graph(create_using: Incomplete | None = None): ...
@_dispatch
def dodecahedral_graph(create_using: Incomplete | None = None): ...
@_dispatch
def frucht_graph(create_using: Incomplete | None = None): ...
@_dispatch
def heawood_graph(create_using: Incomplete | None = None): ...
@_dispatch
def hoffman_singleton_graph(): ...
@_dispatch
def house_graph(create_using: Incomplete | None = None): ...
@_dispatch
def house_x_graph(create_using: Incomplete | None = None): ...
@_dispatch
def icosahedral_graph(create_using: Incomplete | None = None): ...
@_dispatch
def krackhardt_kite_graph(create_using: Incomplete | None = None): ...
@_dispatch
def moebius_kantor_graph(create_using: Incomplete | None = None): ...
@_dispatch
def octahedral_graph(create_using: Incomplete | None = None): ...
@_dispatch
def pappus_graph(): ...
@_dispatch
def petersen_graph(create_using: Incomplete | None = None): ...
@_dispatch
def sedgewick_maze_graph(create_using: Incomplete | None = None): ...
@_dispatch
def tetrahedral_graph(create_using: Incomplete | None = None): ...
@_dispatch
def truncated_cube_graph(create_using: Incomplete | None = None): ...
@_dispatch
def truncated_tetrahedron_graph(create_using: Incomplete | None = None): ...
@_dispatch
def tutte_graph(create_using: Incomplete | None = None): ...



from dataclasses import dataclass

from seaborn._marks.base import (
    MappableBool,
    MappableColor,
    MappableFloat,
    MappableString,
    MappableStyle,
    Mark,
    document_properties,
)

class DotBase(Mark): ...

@document_properties
@dataclass
class Dot(DotBase):
    marker: MappableString = ...
    pointsize: MappableFloat = ...
    stroke: MappableFloat = ...
    color: MappableColor = ...
    alpha: MappableFloat = ...
    fill: MappableBool = ...
    edgecolor: MappableColor = ...
    edgealpha: MappableFloat = ...
    edgewidth: MappableFloat = ...
    edgestyle: MappableStyle = ...

@document_properties
@dataclass
class Dots(DotBase):
    marker: MappableString = ...
    pointsize: MappableFloat = ...
    stroke: MappableFloat = ...
    color: MappableColor = ...
    alpha: MappableFloat = ...
    fill: MappableBool = ...
    fillcolor: MappableColor = ...
    fillalpha: MappableFloat = ...



from _typeshed import SliceableBuffer
from typing import Final, Literal

from Xlib.protocol import display, rq

class DisplayError(Exception):
    display: object
    def __init__(self, display: object) -> None: ...

class DisplayNameError(DisplayError): ...

class DisplayConnectionError(DisplayError):
    display: object
    msg: object
    def __init__(self, display: object, msg: object) -> None: ...

class ConnectionClosedError(Exception):
    whom: object
    def __init__(self, whom: object) -> None: ...

class XauthError(Exception): ...
class XNoAuthError(Exception): ...
class ResourceIDError(Exception): ...

class XError(rq.GetAttrData, Exception):
    def __init__(self, display: display.Display, data: SliceableBuffer) -> None: ...

class XResourceError(XError): ...
class BadRequest(XError): ...
class BadValue(XError): ...
class BadWindow(XResourceError): ...
class BadPixmap(XResourceError): ...
class BadAtom(XError): ...
class BadCursor(XResourceError): ...
class BadFont(XResourceError): ...
class BadMatch(XError): ...
class BadDrawable(XResourceError): ...
class BadAccess(XError): ...
class BadAlloc(XError): ...
class BadColor(XResourceError): ...
class BadGC(XResourceError): ...
class BadIDChoice(XResourceError): ...
class BadName(XError): ...
class BadLength(XError): ...
class BadImplementation(XError): ...

xerror_class: Final[dict[int, type[XError]]]

class CatchError:
    error_types: tuple[type[XError], ...]
    error: XError | None
    request: rq.Request | None
    def __init__(self, *errors: type[XError]) -> None: ...
    def __call__(self, error: XError, request: rq.Request | None) -> Literal[0, 1]: ...
    def get_error(self) -> XError | None: ...
    def get_request(self) -> rq.Request | None: ...
    def reset(self) -> None: ...



from _typeshed import Incomplete

from braintree.resource import Resource
from braintree.subscription import Subscription

class PayPalAccount(Resource):
    @staticmethod
    def find(paypal_account_token): ...
    @staticmethod
    def delete(paypal_account_token): ...
    @staticmethod
    def update(paypal_account_token, params: Incomplete | None = None): ...
    @staticmethod
    def signature(): ...
    subscriptions: list[Subscription]
    def __init__(self, gateway, attributes) -> None: ...



import sys
from _typeshed import Incomplete
from typing import ClassVar, Literal

from ._imaging import _PixelAccessor
from .ImageFile import ImageFile

split: Incomplete
field: Incomplete
if sys.platform == "win32":
    gs_binary: Literal["gswin32c", "gswin64c", "gs", False] | None
    gs_windows_binary: Literal["gswin32c", "gswin64c", "gs", False] | None
else:
    gs_binary: Literal["gs", False] | None
    gs_windows_binary: None

def has_ghostscript(): ...
def Ghostscript(tile, size, fp, scale: int = 1, transparency: bool = False): ...

class PSFile:
    fp: Incomplete
    char: Incomplete
    def __init__(self, fp) -> None: ...
    def seek(self, offset, whence=0) -> None: ...
    def readline(self): ...

class EpsImageFile(ImageFile):
    format: ClassVar[Literal["EPS"]]
    format_description: ClassVar[str]
    mode_map: Incomplete
    im: Incomplete
    tile: Incomplete
    def load(self, scale: int = 1, transparency: bool = False) -> _PixelAccessor: ...
    def load_seek(self, *args, **kwargs) -> None: ...



import codecs
import sys
import threading
from _typeshed import Unused
from collections.abc import Iterable
from typing import Any, BinaryIO, TextIO
from typing_extensions import Self

from serial import Serial

def key_description(character: str) -> str: ...

class ConsoleBase:
    byte_output: BinaryIO
    output: codecs.StreamWriter | TextIO
    def __init__(self) -> None: ...
    def setup(self) -> None: ...
    def cleanup(self) -> None: ...
    def getkey(self) -> None: ...
    def write_bytes(self, byte_string: bytes) -> None: ...
    def write(self, text: str) -> None: ...
    def cancel(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: Unused, **kwargs: Unused) -> None: ...

if sys.platform == "win32":
    class Out:
        fd: int
        def __init__(self, fd: int) -> None: ...
        def flush(self) -> None: ...
        def write(self, s: bytes) -> None: ...

    class Console(ConsoleBase):
        fncodes: dict[str, str]
        navcodes: dict[str, str]
        def __del__(self) -> None: ...

else:
    class Console(ConsoleBase):
        fd: int
        old: list[Any]  # return type of termios.tcgetattr()
        enc_stdin: TextIO

class Transform:
    def rx(self, text: str) -> str: ...
    def tx(self, text: str) -> str: ...
    def echo(self, text: str) -> str: ...

class CRLF(Transform): ...
class CR(Transform): ...
class LF(Transform): ...

class NoTerminal(Transform):
    REPLACEMENT_MAP: dict[int, int]

class NoControls(NoTerminal):
    REPLACEMENT_MAP: dict[int, int]

class Printable(Transform): ...

class Colorize(Transform):
    input_color: str
    echo_color: str

class DebugIO(Transform): ...

EOL_TRANSFORMATIONS: dict[str, type[Transform]]
TRANSFORMATIONS: dict[str, type[Transform]]

def ask_for_port() -> str: ...

class Miniterm:
    console: Console
    serial: Serial
    echo: bool
    raw: bool
    input_encoding: str
    output_encoding: str
    eol: str
    filters: Iterable[str]
    exit_character: str
    menu_character: str
    alive: bool | None
    receiver_thread: threading.Thread | None
    rx_decoder: codecs.IncrementalDecoder | None
    tx_decoder: codecs.IncrementalDecoder | None
    tx_encoder: codecs.IncrementalEncoder | None
    def __init__(self, serial_instance: Serial, echo: bool = False, eol: str = "crlf", filters: Iterable[str] = ()) -> None: ...
    transmitter_thread: threading.Thread
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def join(self, transmit_only: bool = False) -> None: ...
    def close(self) -> None: ...
    tx_transformations: list[Transform]
    rx_transformations: list[Transform]
    def update_transformations(self) -> None: ...
    def set_rx_encoding(self, encoding: str, errors: str = "replace") -> None: ...
    def set_tx_encoding(self, encoding: str, errors: str = "replace") -> None: ...
    def dump_port_settings(self) -> None: ...
    def reader(self) -> None: ...
    def writer(self) -> None: ...
    def handle_menu_key(self, c: str) -> None: ...
    def upload_file(self) -> None: ...
    def change_filter(self) -> None: ...
    def change_encoding(self) -> None: ...
    def change_baudrate(self) -> None: ...
    def change_port(self) -> None: ...
    def suspend_port(self) -> None: ...
    def get_help_text(self) -> str: ...

def main(
    default_port: str | None = None, default_baudrate: int = 9600, default_rts: int | None = None, default_dtr: int | None = None
) -> None: ...



from braintree.search import Search

class TransactionSearch:
    billing_company: Search.TextNodeBuilder
    billing_country_name: Search.TextNodeBuilder
    billing_extended_address: Search.TextNodeBuilder
    billing_first_name: Search.TextNodeBuilder
    billing_last_name: Search.TextNodeBuilder
    billing_locality: Search.TextNodeBuilder
    billing_postal_code: Search.TextNodeBuilder
    billing_region: Search.TextNodeBuilder
    billing_street_address: Search.TextNodeBuilder
    credit_card_cardholder_name: Search.TextNodeBuilder
    currency: Search.TextNodeBuilder
    customer_company: Search.TextNodeBuilder
    customer_email: Search.TextNodeBuilder
    customer_fax: Search.TextNodeBuilder
    customer_first_name: Search.TextNodeBuilder
    customer_id: Search.TextNodeBuilder
    customer_last_name: Search.TextNodeBuilder
    customer_phone: Search.TextNodeBuilder
    customer_website: Search.TextNodeBuilder
    id: Search.TextNodeBuilder
    order_id: Search.TextNodeBuilder
    payment_method_token: Search.TextNodeBuilder
    processor_authorization_code: Search.TextNodeBuilder
    europe_bank_account_iban: Search.TextNodeBuilder
    settlement_batch_id: Search.TextNodeBuilder
    shipping_company: Search.TextNodeBuilder
    shipping_country_name: Search.TextNodeBuilder
    shipping_extended_address: Search.TextNodeBuilder
    shipping_first_name: Search.TextNodeBuilder
    shipping_last_name: Search.TextNodeBuilder
    shipping_locality: Search.TextNodeBuilder
    shipping_postal_code: Search.TextNodeBuilder
    shipping_region: Search.TextNodeBuilder
    shipping_street_address: Search.TextNodeBuilder
    paypal_payer_email: Search.TextNodeBuilder
    paypal_payment_id: Search.TextNodeBuilder
    paypal_authorization_id: Search.TextNodeBuilder
    sepa_debit_paypal_v2_order_id: Search.TextNodeBuilder
    credit_card_unique_identifier: Search.TextNodeBuilder
    store_id: Search.TextNodeBuilder
    credit_card_expiration_date: Search.EqualityNodeBuilder
    credit_card_number: Search.PartialMatchNodeBuilder
    user: Search.MultipleValueNodeBuilder
    ids: Search.MultipleValueNodeBuilder
    merchant_account_id: Search.MultipleValueNodeBuilder
    payment_instrument_type: Search.MultipleValueNodeBuilder
    store_ids: Search.MultipleValueNodeBuilder
    created_using: Search.MultipleValueNodeBuilder
    credit_card_card_type: Search.MultipleValueNodeBuilder
    credit_card_customer_location: Search.MultipleValueNodeBuilder
    debit_network: Search.MultipleValueNodeBuilder
    source: Search.MultipleValueNodeBuilder
    status: Search.MultipleValueNodeBuilder
    type: Search.MultipleValueNodeBuilder
    refund: Search.KeyValueNodeBuilder
    amount: Search.RangeNodeBuilder
    authorization_expired_at: Search.RangeNodeBuilder
    authorized_at: Search.RangeNodeBuilder
    created_at: Search.RangeNodeBuilder
    disbursement_date: Search.RangeNodeBuilder
    dispute_date: Search.RangeNodeBuilder
    failed_at: Search.RangeNodeBuilder
    gateway_rejected_at: Search.RangeNodeBuilder
    processor_declined_at: Search.RangeNodeBuilder
    settled_at: Search.RangeNodeBuilder
    submitted_for_settlement_at: Search.RangeNodeBuilder
    voided_at: Search.RangeNodeBuilder
    ach_return_responses_created_at: Search.RangeNodeBuilder
    reason_code: Search.MultipleValueNodeBuilder



from _typeshed import Incomplete

from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Health(SystemBackendMixin):
    def read_health_status(
        self,
        standby_ok: Incomplete | None = None,
        active_code: Incomplete | None = None,
        standby_code: Incomplete | None = None,
        dr_secondary_code: Incomplete | None = None,
        performance_standby_code: Incomplete | None = None,
        sealed_code: Incomplete | None = None,
        uninit_code: Incomplete | None = None,
        method: str = "HEAD",
    ): ...



from _typeshed import Incomplete
from logging import Logger
from typing import Any, Literal, Protocol, TypedDict, overload
from typing_extensions import TypeAlias

import requests
from oauthlib.oauth2 import Client
from requests.cookies import RequestsCookieJar

_Token: TypeAlias = dict[str, Incomplete]  # oauthlib.oauth2.Client.token

class _AccessTokenResponseHook(Protocol):
    def __call__(self, response: requests.Response, /) -> requests.Response: ...

class _RefreshTokenResponseHook(Protocol):
    def __call__(self, response: requests.Response, /) -> requests.Response: ...

class _ProtectedRequestHook(Protocol):
    def __call__(
        self, url: Incomplete, headers: Incomplete, data: Incomplete, /
    ) -> tuple[Incomplete, Incomplete, Incomplete]: ...

class _ComplianceHooks(TypedDict):
    access_token_response: set[_AccessTokenResponseHook]
    refresh_token_response: set[_RefreshTokenResponseHook]
    protected_request: set[_ProtectedRequestHook]

log: Logger

class TokenUpdated(Warning):
    token: Incomplete
    def __init__(self, token) -> None: ...

class OAuth2Session(requests.Session):
    redirect_uri: Incomplete
    state: Incomplete
    auto_refresh_url: str | None
    auto_refresh_kwargs: dict[str, Any]
    token_updater: Incomplete
    compliance_hook: _ComplianceHooks
    def __init__(
        self,
        client_id: Incomplete | None = None,
        client: Client | None = None,
        auto_refresh_url: str | None = None,
        auto_refresh_kwargs: dict[str, Any] | None = None,
        scope: Incomplete | None = None,
        redirect_uri: Incomplete | None = None,
        token: Incomplete | None = None,
        state: Incomplete | None = None,
        token_updater: Incomplete | None = None,
        pkce: Incomplete | None = None,
        **kwargs,
    ) -> None: ...
    @property
    def scope(self) -> Incomplete | None: ...  # oauthlib.oauth2.Client.scope
    @scope.setter
    def scope(self, value: Incomplete | None) -> None: ...
    def new_state(self): ...
    @property
    def client_id(self) -> Incomplete | None: ...  # oauthlib.oauth2.Client.client_id
    @client_id.setter
    def client_id(self, value: Incomplete | None) -> None: ...
    @client_id.deleter
    def client_id(self) -> None: ...
    @property
    def token(self): ...  # oauthlib.oauth2.Client.token
    @token.setter
    def token(self, value) -> None: ...
    @property
    def access_token(self): ...  # oauthlib.oauth2.Client.access_token
    @access_token.setter
    def access_token(self, value) -> None: ...
    @access_token.deleter
    def access_token(self) -> None: ...
    @property
    def authorized(self) -> bool: ...
    def authorization_url(self, url: str, state: Incomplete | None = None, **kwargs) -> tuple[str, str]: ...
    def fetch_token(
        self,
        token_url: str,
        code: Incomplete | None = None,
        authorization_response: Incomplete | None = None,
        body: str = "",
        auth: Incomplete | None = None,
        username: Incomplete | None = None,
        password: Incomplete | None = None,
        method: str = "POST",
        force_querystring: bool = False,
        timeout: Incomplete | None = None,
        headers: Incomplete | None = None,
        verify: bool | None = None,
        proxies: Incomplete | None = None,
        include_client_id: Incomplete | None = None,
        client_secret: Incomplete | None = None,
        cert: Incomplete | None = None,
        **kwargs,
    ) -> _Token: ...
    def token_from_fragment(self, authorization_response: str) -> _Token: ...
    def refresh_token(
        self,
        token_url: str,
        refresh_token: Incomplete | None = None,
        body: str = "",
        auth: Incomplete | None = None,
        timeout: Incomplete | None = None,
        headers: Incomplete | None = None,
        verify: bool | None = None,
        proxies: Incomplete | None = None,
        **kwargs,
    ) -> _Token: ...
    def request(  # type: ignore[override]
        self,
        method: str | bytes,
        url: str | bytes,
        data: requests.sessions._Data | None = None,
        headers: requests.sessions._HeadersUpdateMapping | None = None,
        withhold_token: bool = False,
        client_id: Incomplete | None = None,
        client_secret: Incomplete | None = None,
        files: requests.sessions._Files | None = None,
        *,
        params: requests.sessions._Params | None = None,
        cookies: None | RequestsCookieJar | requests.sessions._TextMapping = None,
        auth: requests.sessions._Auth | None = None,
        timeout: requests.sessions._Timeout | None = None,
        allow_redirects: bool = True,
        proxies: requests.sessions._TextMapping | None = None,
        hooks: requests.sessions._HooksInput | None = None,
        stream: bool | None = None,
        verify: requests.sessions._Verify | None = None,
        cert: requests.sessions._Cert | None = None,
        json: Incomplete | None = None,
    ) -> requests.Response: ...
    @overload
    def register_compliance_hook(self, hook_type: Literal["access_token_response"], hook: _AccessTokenResponseHook) -> None: ...
    @overload
    def register_compliance_hook(self, hook_type: Literal["refresh_token_response"], hook: _RefreshTokenResponseHook) -> None: ...
    @overload
    def register_compliance_hook(self, hook_type: Literal["protected_request"], hook: _ProtectedRequestHook) -> None: ...



from _typeshed import Incomplete, SupportsRichComparison
from collections import deque
from collections.abc import Callable, Container, Iterable, Iterator, MutableMapping, Sequence
from typing_extensions import Self, TypeAlias, deprecated

from jsonschema import _utils, protocols
from jsonschema._types import TypeChecker

_RelevanceFuncType: TypeAlias = Callable[[ValidationError], SupportsRichComparison]

WEAK_MATCHES: frozenset[str]
STRONG_MATCHES: frozenset[str]

class _Error(Exception):
    message: str
    path: deque[str | int]
    relative_path: deque[str | int]
    schema_path: deque[str | int]
    relative_schema_path: deque[str | int]
    context: list[ValidationError] | None
    cause: Exception | None
    validator: protocols.Validator | None
    validator_value: Incomplete
    instance: Incomplete
    schema: Incomplete
    parent: _Error | None
    def __init__(
        self,
        message: str,
        validator: _utils.Unset | None | protocols.Validator = ...,
        path: Sequence[str | int] = (),
        cause: Incomplete | None = None,
        context: Sequence[ValidationError] = (),
        validator_value=...,
        instance=...,
        schema=...,
        schema_path: Sequence[str | int] = (),
        parent: _Error | None = None,
        type_checker: _utils.Unset | TypeChecker = ...,
    ) -> None: ...
    @classmethod
    def create_from(cls, other: _Error) -> Self: ...
    @property
    def absolute_path(self) -> Sequence[str | int]: ...
    @property
    def absolute_schema_path(self) -> Sequence[str | int]: ...
    @property
    def json_path(self) -> str: ...
    # TODO: this type could be made more precise using TypedDict to
    # enumerate the types of the members
    def _contents(self) -> dict[str, Incomplete]: ...

class ValidationError(_Error): ...
class SchemaError(_Error): ...

class RefResolutionError(Exception):
    def __init__(self, cause: str) -> None: ...

class UndefinedTypeCheck(Exception):
    type: Incomplete
    def __init__(self, type) -> None: ...

class UnknownType(Exception):
    type: Incomplete
    instance: Incomplete
    schema: Incomplete
    def __init__(self, type, instance, schema) -> None: ...

class FormatError(Exception):
    message: Incomplete
    cause: Incomplete
    def __init__(self, message, cause: Incomplete | None = None) -> None: ...

class ErrorTree:
    errors: MutableMapping[str, ValidationError]
    def __init__(self, errors: Iterable[ValidationError] = ()) -> None: ...
    def __contains__(self, index: object) -> bool: ...
    def __getitem__(self, index): ...
    @deprecated("ErrorTree.__setitem__ is deprecated without replacement.")
    def __setitem__(self, index: str | int, value: ErrorTree) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...
    @property
    def total_errors(self): ...

def by_relevance(weak: Container[str] = ..., strong: Container[str] = ...) -> _RelevanceFuncType: ...

relevance: _RelevanceFuncType

def best_match(errors: Iterable[ValidationError], key: _RelevanceFuncType = ...): ...



from _typeshed import Incomplete
from abc import abstractmethod
from collections.abc import Callable, Iterable
from typing import ClassVar
from typing_extensions import Self

from .dist import Distribution

class Command:
    distribution: Distribution
    sub_commands: ClassVar[list[tuple[str, Callable[[Self], bool] | None]]]
    def __init__(self, dist: Distribution) -> None: ...
    def ensure_finalized(self) -> None: ...
    @abstractmethod
    def initialize_options(self) -> None: ...
    @abstractmethod
    def finalize_options(self) -> None: ...
    @abstractmethod
    def run(self) -> None: ...
    def announce(self, msg: str, level: int = ...) -> None: ...
    def debug_print(self, msg: str) -> None: ...
    def ensure_string(self, option: str, default: str | None = ...) -> None: ...
    def ensure_string_list(self, option: str | list[str]) -> None: ...
    def ensure_filename(self, option: str) -> None: ...
    def ensure_dirname(self, option: str) -> None: ...
    def get_command_name(self) -> str: ...
    def set_undefined_options(self, src_cmd: str, *option_pairs: tuple[str, str]) -> None: ...
    def get_finalized_command(self, command: str, create: int = ...) -> Command: ...
    def reinitialize_command(self, command: Command | str, reinit_subcommands: int = ...) -> Command: ...
    def run_command(self, command: str) -> None: ...
    def get_sub_commands(self) -> list[str]: ...
    def warn(self, msg: str) -> None: ...
    def execute(
        self, func: Callable[..., object], args: Iterable[Incomplete], msg: str | None = ..., level: int = ...
    ) -> None: ...
    def mkpath(self, name: str, mode: int = ...) -> None: ...
    def copy_file(
        self,
        infile: str,
        outfile: str,
        preserve_mode: int = ...,
        preserve_times: int = ...,
        link: str | None = ...,
        level: int = ...,
    ) -> tuple[str, bool]: ...  # level is not used
    def copy_tree(
        self,
        infile: str,
        outfile: str,
        preserve_mode: int = ...,
        preserve_times: int = ...,
        preserve_symlinks: int = ...,
        level: int = ...,
    ) -> list[str]: ...  # level is not used
    def move_file(self, src: str, dst: str, level: int = ...) -> str: ...  # level is not used
    def spawn(self, cmd: Iterable[str], search_path: int = ..., level: int = ...) -> None: ...  # level is not used
    def make_archive(
        self,
        base_name: str,
        format: str,
        root_dir: str | None = ...,
        base_dir: str | None = ...,
        owner: str | None = ...,
        group: str | None = ...,
    ) -> str: ...
    def make_file(
        self,
        infiles: str | list[str] | tuple[str, ...],
        outfile: str,
        func: Callable[..., object],
        args: list[Incomplete],
        exec_msg: str | None = ...,
        skip_msg: str | None = ...,
        level: int = ...,
    ) -> None: ...  # level is not used



from collections.abc import Iterator

from ._pygit2 import Oid, Reference
from .enums import ReferenceFilter
from .repository import BaseRepository

class References:
    def __init__(self, repository: BaseRepository) -> None: ...
    def __getitem__(self, name: str) -> Reference: ...
    def get(self, key: str) -> Reference | None: ...
    def __iter__(self) -> Iterator[Reference]: ...
    def iterator(self, references_return_type: ReferenceFilter = ...) -> Iterator[Reference]: ...
    def create(self, name: str, target: Oid | str, force: bool = False) -> Reference: ...
    def delete(self, name: str) -> None: ...
    def __contains__(self, name: str) -> bool: ...
    @property
    def objects(self) -> list[Reference]: ...
    def compress(self) -> None: ...



from _typeshed import Incomplete

class DictItem:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(self, type: Incomplete | None = None, key: Incomplete | None = None, val: Incomplete | None = None) -> None: ...
    @property
    def type(self): ...
    @type.setter
    def type(self, type) -> None: ...
    @property
    def key(self): ...
    @key.setter
    def key(self, key) -> None: ...
    @property
    def val(self): ...
    @val.setter
    def val(self, val) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete

class QueryVariablePropertiesValues:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(self, query: Incomplete | None = None, language: Incomplete | None = None) -> None: ...
    @property
    def query(self): ...
    @query.setter
    def query(self, query) -> None: ...
    @property
    def language(self): ...
    @language.setter
    def language(self, language) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from collections import defaultdict
from typing import TypeVar

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")

class BoundDictionary(defaultdict[_KT, _VT]):
    reference: str | None
    def __init__(self, reference: str | None = None, *args, **kw) -> None: ...



from _typeshed import Incomplete

from braintree.address import Address
from braintree.amex_express_checkout_card import AmexExpressCheckoutCard
from braintree.android_pay_card import AndroidPayCard
from braintree.apple_pay_card import ApplePayCard
from braintree.credit_card import CreditCard
from braintree.europe_bank_account import EuropeBankAccount
from braintree.masterpass_card import MasterpassCard
from braintree.paypal_account import PayPalAccount
from braintree.resource import Resource
from braintree.samsung_pay_card import SamsungPayCard
from braintree.us_bank_account import UsBankAccount
from braintree.venmo_account import VenmoAccount
from braintree.visa_checkout_card import VisaCheckoutCard

class Customer(Resource):
    @staticmethod
    def all(): ...
    @staticmethod
    def create(params: Incomplete | None = None): ...
    @staticmethod
    def delete(customer_id): ...
    @staticmethod
    def find(customer_id, association_filter_id: Incomplete | None = None): ...
    @staticmethod
    def search(*query): ...
    @staticmethod
    def update(customer_id, params: Incomplete | None = None): ...
    @staticmethod
    def create_signature(): ...
    @staticmethod
    def update_signature(): ...
    payment_methods: list[Resource]
    credit_cards: list[CreditCard]
    addresses: list[Address]
    paypal_accounts: list[PayPalAccount]
    apple_pay_cards: list[ApplePayCard]
    android_pay_cards: list[AndroidPayCard]
    amex_express_checkout_cards: list[AmexExpressCheckoutCard]
    europe_bank_accounts: list[EuropeBankAccount]
    venmo_accounts: list[VenmoAccount]
    us_bank_accounts: list[UsBankAccount]
    visa_checkout_cards: list[VisaCheckoutCard]
    masterpass_cards: list[MasterpassCard]
    samsung_pay_cards: list[SamsungPayCard]
    def __init__(self, gateway, attributes) -> None: ...



from _typeshed import Incomplete

from antlr4.CommonTokenStream import CommonTokenStream as CommonTokenStream
from antlr4.Token import Token as Token

class TokenStreamRewriter:
    DEFAULT_PROGRAM_NAME: str
    PROGRAM_INIT_SIZE: int
    MIN_TOKEN_INDEX: int
    tokens: Incomplete
    programs: Incomplete
    lastRewriteTokenIndexes: Incomplete
    def __init__(self, tokens) -> None: ...
    def getTokenStream(self): ...
    def rollback(self, instruction_index, program_name) -> None: ...
    def deleteProgram(self, program_name="default") -> None: ...
    def insertAfterToken(self, token, text, program_name="default") -> None: ...
    def insertAfter(self, index, text, program_name="default") -> None: ...
    def insertBeforeIndex(self, index, text) -> None: ...
    def insertBeforeToken(self, token, text, program_name="default") -> None: ...
    def insertBefore(self, program_name, index, text) -> None: ...
    def replaceIndex(self, index, text) -> None: ...
    def replaceRange(self, from_idx, to_idx, text) -> None: ...
    def replaceSingleToken(self, token, text) -> None: ...
    def replaceRangeTokens(self, from_token, to_token, text, program_name="default") -> None: ...
    def replace(self, program_name, from_idx, to_idx, text) -> None: ...
    def deleteToken(self, token) -> None: ...
    def deleteIndex(self, index) -> None: ...
    def delete(self, program_name, from_idx, to_idx) -> None: ...
    def lastRewriteTokenIndex(self, program_name="default"): ...
    def setLastRewriteTokenIndex(self, program_name, i) -> None: ...
    def getProgram(self, program_name): ...
    def getDefaultText(self): ...
    def getText(self, program_name, start: int, stop: int): ...

    class RewriteOperation:
        tokens: Incomplete
        index: Incomplete
        text: Incomplete
        instructionIndex: int
        def __init__(self, tokens, index, text: str = "") -> None: ...
        def execute(self, buf): ...

    class InsertBeforeOp(RewriteOperation):
        def __init__(self, tokens, index, text: str = "") -> None: ...
        def execute(self, buf): ...

    class InsertAfterOp(InsertBeforeOp): ...

    class ReplaceOp(RewriteOperation):
        last_index: Incomplete
        def __init__(self, from_idx, to_idx, tokens, text) -> None: ...
        def execute(self, buf): ...



from commonmark.blocks import Parser as Parser
from commonmark.dump import dumpAST as dumpAST, dumpJSON as dumpJSON
from commonmark.main import commonmark as commonmark
from commonmark.render.html import HtmlRenderer as HtmlRenderer
from commonmark.render.rst import ReStructuredTextRenderer as ReStructuredTextRenderer



from _typeshed import Incomplete

from .Image import ImageTransformHandler

class Transform(ImageTransformHandler):
    data: Incomplete
    def __init__(self, data) -> None: ...
    def getdata(self): ...
    def transform(self, size, image, **options): ...

class AffineTransform(Transform):
    method: Incomplete

class ExtentTransform(Transform):
    method: Incomplete

class QuadTransform(Transform):
    method: Incomplete

class MeshTransform(Transform):
    method: Incomplete



from typing import Final

XK_overline: Final = 0x47E
XK_kana_fullstop: Final = 0x4A1
XK_kana_openingbracket: Final = 0x4A2
XK_kana_closingbracket: Final = 0x4A3
XK_kana_comma: Final = 0x4A4
XK_kana_conjunctive: Final = 0x4A5
XK_kana_middledot: Final = 0x4A5
XK_kana_WO: Final = 0x4A6
XK_kana_a: Final = 0x4A7
XK_kana_i: Final = 0x4A8
XK_kana_u: Final = 0x4A9
XK_kana_e: Final = 0x4AA
XK_kana_o: Final = 0x4AB
XK_kana_ya: Final = 0x4AC
XK_kana_yu: Final = 0x4AD
XK_kana_yo: Final = 0x4AE
XK_kana_tsu: Final = 0x4AF
XK_kana_tu: Final = 0x4AF
XK_prolongedsound: Final = 0x4B0
XK_kana_A: Final = 0x4B1
XK_kana_I: Final = 0x4B2
XK_kana_U: Final = 0x4B3
XK_kana_E: Final = 0x4B4
XK_kana_O: Final = 0x4B5
XK_kana_KA: Final = 0x4B6
XK_kana_KI: Final = 0x4B7
XK_kana_KU: Final = 0x4B8
XK_kana_KE: Final = 0x4B9
XK_kana_KO: Final = 0x4BA
XK_kana_SA: Final = 0x4BB
XK_kana_SHI: Final = 0x4BC
XK_kana_SU: Final = 0x4BD
XK_kana_SE: Final = 0x4BE
XK_kana_SO: Final = 0x4BF
XK_kana_TA: Final = 0x4C0
XK_kana_CHI: Final = 0x4C1
XK_kana_TI: Final = 0x4C1
XK_kana_TSU: Final = 0x4C2
XK_kana_TU: Final = 0x4C2
XK_kana_TE: Final = 0x4C3
XK_kana_TO: Final = 0x4C4
XK_kana_NA: Final = 0x4C5
XK_kana_NI: Final = 0x4C6
XK_kana_NU: Final = 0x4C7
XK_kana_NE: Final = 0x4C8
XK_kana_NO: Final = 0x4C9
XK_kana_HA: Final = 0x4CA
XK_kana_HI: Final = 0x4CB
XK_kana_FU: Final = 0x4CC
XK_kana_HU: Final = 0x4CC
XK_kana_HE: Final = 0x4CD
XK_kana_HO: Final = 0x4CE
XK_kana_MA: Final = 0x4CF
XK_kana_MI: Final = 0x4D0
XK_kana_MU: Final = 0x4D1
XK_kana_ME: Final = 0x4D2
XK_kana_MO: Final = 0x4D3
XK_kana_YA: Final = 0x4D4
XK_kana_YU: Final = 0x4D5
XK_kana_YO: Final = 0x4D6
XK_kana_RA: Final = 0x4D7
XK_kana_RI: Final = 0x4D8
XK_kana_RU: Final = 0x4D9
XK_kana_RE: Final = 0x4DA
XK_kana_RO: Final = 0x4DB
XK_kana_WA: Final = 0x4DC
XK_kana_N: Final = 0x4DD
XK_voicedsound: Final = 0x4DE
XK_semivoicedsound: Final = 0x4DF
XK_kana_switch: Final = 0xFF7E



from _typeshed import Incomplete, Unused
from typing import ClassVar, Literal

from openpyxl.chart.axis import ChartLines, NumericAxis, TextAxis
from openpyxl.chart.label import DataLabelList
from openpyxl.chart.updown_bars import UpDownBars
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList

from ._chart import ChartBase

class StockChart(ChartBase):
    tagname: ClassVar[str]
    ser: Incomplete
    dLbls: Typed[DataLabelList, Literal[True]]
    dataLabels: Alias
    dropLines: Typed[ChartLines, Literal[True]]
    hiLowLines: Typed[ChartLines, Literal[True]]
    upDownBars: Typed[UpDownBars, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    x_axis: Typed[TextAxis, Literal[False]]
    y_axis: Typed[NumericAxis, Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        ser=(),
        dLbls: DataLabelList | None = None,
        dropLines: ChartLines | None = None,
        hiLowLines: ChartLines | None = None,
        upDownBars: UpDownBars | None = None,
        extLst: Unused = None,
        **kw,
    ) -> None: ...



from collections.abc import Callable, Generator, Iterable, Sequence
from queue import Queue
from threading import Event as _UninterruptibleEvent
from typing_extensions import TypeAlias

from ._canonical_names import all_modifiers as all_modifiers, sided_modifiers as sided_modifiers
from ._keyboard_event import KEY_DOWN as KEY_DOWN, KEY_UP as KEY_UP, KeyboardEvent as KeyboardEvent

_Key: TypeAlias = int | str
_ScanCodeList: TypeAlias = list[int] | tuple[int, ...]
_ParseableHotkey: TypeAlias = _Key | list[int | _ScanCodeList] | tuple[int | _ScanCodeList, ...]
_Callback: TypeAlias = Callable[[KeyboardEvent], bool | None] | Callable[[], bool | None]
# mypy doesn't support PEP 646's TypeVarTuple yet: https://github.com/python/mypy/issues/12280
# _Ts = TypeVarTuple("_Ts")
_Ts: TypeAlias = tuple[object, ...]

version: str

class _Event(_UninterruptibleEvent):
    def wait(self) -> None: ...  # type: ignore[override]  # Actual implementation

def is_modifier(key: _Key | None) -> bool: ...
def key_to_scan_codes(key: _ParseableHotkey, error_if_missing: bool = True) -> tuple[int, ...]: ...
def parse_hotkey(hotkey: _ParseableHotkey) -> tuple[tuple[tuple[int, ...], ...], ...]: ...
def send(hotkey: _ParseableHotkey, do_press: bool = True, do_release: bool = True) -> None: ...

press_and_release = send

def press(hotkey: _ParseableHotkey) -> None: ...
def release(hotkey: _ParseableHotkey) -> None: ...

# is_pressed cannot check multi-step hotkeys, so not using _ParseableHotkey

def is_pressed(hotkey: _Key | _ScanCodeList) -> bool: ...
def call_later(fn: Callable[..., None], args: _Ts = (), delay: float = 0.001) -> None: ...
def hook(callback: _Callback, suppress: bool = False, on_remove: Callable[[], None] = ...) -> Callable[[], None]: ...
def on_press(callback: _Callback, suppress: bool = False) -> Callable[[], None]: ...
def on_release(callback: _Callback, suppress: bool = False) -> Callable[[], None]: ...
def hook_key(key: _ParseableHotkey, callback: _Callback, suppress: bool = False) -> Callable[[], None]: ...
def on_press_key(key: _ParseableHotkey, callback: _Callback, suppress: bool = False) -> Callable[[], None]: ...
def on_release_key(key: _ParseableHotkey, callback: _Callback, suppress: bool = False) -> Callable[[], None]: ...
def unhook(remove: _Callback) -> None: ...

unhook_key = unhook

def unhook_all() -> None: ...
def block_key(key: _ParseableHotkey) -> Callable[[], None]: ...

unblock_key = unhook_key

def remap_key(src: _ParseableHotkey, dst: _ParseableHotkey) -> Callable[[], None]: ...

unremap_key = unhook_key

def parse_hotkey_combinations(hotkey: _ParseableHotkey) -> tuple[tuple[tuple[int, ...], ...], ...]: ...
def add_hotkey(
    hotkey: _ParseableHotkey,
    callback: Callable[..., bool | None],
    args: _Ts = (),
    suppress: bool = False,
    timeout: float = 1,
    trigger_on_release: bool = False,
) -> Callable[[], None]: ...

register_hotkey = add_hotkey

def remove_hotkey(hotkey_or_callback: _ParseableHotkey | _Callback) -> None: ...

unregister_hotkey = remove_hotkey
clear_hotkey = remove_hotkey

def unhook_all_hotkeys() -> None: ...

unregister_all_hotkeys = unhook_all_hotkeys
remove_all_hotkeys = unhook_all_hotkeys
clear_all_hotkeys = unhook_all_hotkeys

def remap_hotkey(
    src: _ParseableHotkey, dst: _ParseableHotkey, suppress: bool = True, trigger_on_release: bool = False
) -> Callable[[], None]: ...

unremap_hotkey = remove_hotkey

def stash_state() -> list[int]: ...
def restore_state(scan_codes: Iterable[int]) -> None: ...
def restore_modifiers(scan_codes: Iterable[int]) -> None: ...
def write(text: str, delay: float = 0, restore_state_after: bool = True, exact: bool | None = None) -> None: ...
def wait(hotkey: _ParseableHotkey | None = None, suppress: bool = False, trigger_on_release: bool = False) -> None: ...
def get_hotkey_name(names: Iterable[str] | None = None) -> str: ...
def read_event(suppress: bool = False) -> KeyboardEvent: ...
def read_key(suppress: bool = False) -> _Key: ...
def read_hotkey(suppress: bool = True) -> str: ...
def get_typed_strings(events: Iterable[KeyboardEvent], allow_backspace: bool = True) -> Generator[str, None, None]: ...
def start_recording(
    recorded_events_queue: Queue[KeyboardEvent] | None = None,
) -> tuple[Queue[KeyboardEvent], Callable[[], None]]: ...
def stop_recording() -> list[KeyboardEvent]: ...
def record(until: str = "escape", suppress: bool = False, trigger_on_release: bool = False) -> list[KeyboardEvent]: ...
def play(events: Iterable[KeyboardEvent], speed_factor: float = 1.0) -> None: ...

replay = play

def add_word_listener(
    word: str, callback: _Callback, triggers: Sequence[str] = ["space"], match_suffix: bool = False, timeout: float = 2
) -> Callable[[], None]: ...
def remove_word_listener(word_or_handler: str | _Callback) -> None: ...
def add_abbreviation(
    source_text: str, replacement_text: str, match_suffix: bool = False, timeout: float = 2
) -> Callable[[], None]: ...

register_word_listener = add_word_listener
register_abbreviation = add_abbreviation
remove_abbreviation = remove_word_listener



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class TemplatesService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def apply_template(self, template_apply, **kwargs): ...
    def apply_template_with_http_info(self, template_apply, **kwargs): ...
    async def apply_template_async(self, template_apply, **kwargs): ...
    def create_stack(self, **kwargs): ...
    def create_stack_with_http_info(self, **kwargs): ...
    async def create_stack_async(self, **kwargs): ...
    def delete_stack(self, stack_id, org_id, **kwargs): ...
    def delete_stack_with_http_info(self, stack_id, org_id, **kwargs): ...
    async def delete_stack_async(self, stack_id, org_id, **kwargs): ...
    def export_template(self, **kwargs): ...
    def export_template_with_http_info(self, **kwargs): ...
    async def export_template_async(self, **kwargs): ...
    def list_stacks(self, org_id, **kwargs): ...
    def list_stacks_with_http_info(self, org_id, **kwargs): ...
    async def list_stacks_async(self, org_id, **kwargs): ...
    def read_stack(self, stack_id, **kwargs): ...
    def read_stack_with_http_info(self, stack_id, **kwargs): ...
    async def read_stack_async(self, stack_id, **kwargs): ...
    def uninstall_stack(self, stack_id, **kwargs): ...
    def uninstall_stack_with_http_info(self, stack_id, **kwargs): ...
    async def uninstall_stack_async(self, stack_id, **kwargs): ...
    def update_stack(self, stack_id, **kwargs): ...
    def update_stack_with_http_info(self, stack_id, **kwargs): ...
    async def update_stack_async(self, stack_id, **kwargs): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def fast_gnp_random_graph(n, p, seed: Incomplete | None = None, directed: bool = False): ...
@_dispatch
def gnp_random_graph(n, p, seed: Incomplete | None = None, directed: bool = False): ...

binomial_graph = gnp_random_graph
erdos_renyi_graph = gnp_random_graph

@_dispatch
def dense_gnm_random_graph(n, m, seed: Incomplete | None = None): ...
@_dispatch
def gnm_random_graph(n, m, seed: Incomplete | None = None, directed: bool = False): ...
@_dispatch
def newman_watts_strogatz_graph(n, k, p, seed: Incomplete | None = None): ...
@_dispatch
def watts_strogatz_graph(n, k, p, seed: Incomplete | None = None): ...
@_dispatch
def connected_watts_strogatz_graph(n, k, p, tries: int = 100, seed: Incomplete | None = None): ...
@_dispatch
def random_regular_graph(d, n, seed: Incomplete | None = None): ...
@_dispatch
def barabasi_albert_graph(n, m, seed: Incomplete | None = None, initial_graph: Incomplete | None = None): ...
@_dispatch
def dual_barabasi_albert_graph(n, m1, m2, p, seed: Incomplete | None = None, initial_graph: Incomplete | None = None): ...
@_dispatch
def extended_barabasi_albert_graph(n, m, p, q, seed: Incomplete | None = None): ...
@_dispatch
def powerlaw_cluster_graph(n, m, p, seed: Incomplete | None = None): ...
@_dispatch
def random_lobster(n, p1, p2, seed: Incomplete | None = None): ...
@_dispatch
def random_shell_graph(constructor, seed: Incomplete | None = None): ...
@_dispatch
def random_powerlaw_tree(n, gamma: float = 3, seed: Incomplete | None = None, tries: int = 100): ...
@_dispatch
def random_powerlaw_tree_sequence(n, gamma: float = 3, seed: Incomplete | None = None, tries: int = 100): ...
@_dispatch
def random_kernel_graph(n, kernel_integral, kernel_root: Incomplete | None = None, seed: Incomplete | None = None): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def projected_graph(B, nodes, multigraph: bool = False): ...
@_dispatch
def weighted_projected_graph(B, nodes, ratio: bool = False): ...
@_dispatch
def collaboration_weighted_projected_graph(B, nodes): ...
@_dispatch
def overlap_weighted_projected_graph(B, nodes, jaccard: bool = True): ...
@_dispatch
def generic_weighted_projected_graph(B, nodes, weight_function: Incomplete | None = None): ...



import sys
from collections.abc import Callable, Mapping, Sequence
from types import ModuleType
from typing import Any, Protocol, TypeVar
from typing_extensions import TypeAlias

from gevent.hub import Hub
from greenlet import greenlet as greenlet_t

_T = TypeVar("_T")
# FIXME: While it would be nice to import Interface from zope.interface here so the
#        mypy plugin will work correctly for the people that use it, it causes all
#        sorts of issues to reference a module that is not stubbed in typeshed, so
#        for now we punt and just define an alias for Interface and implementer we
#        can get rid of later
Interface: TypeAlias = Any

def implementer(interface: Interface, /) -> Callable[[_T], _T]: ...

# this is copied from types-psutil, it would be nice if we could just import this
# but it doesn't seem like we can...
if sys.platform == "linux":
    from psutil._pslinux import pmem
elif sys.platform == "darwin":
    from psutil._psosx import pmem
elif sys.platform == "win32":
    from psutil._pswindows import pmem
else:
    class pmem(Any): ...

subscribers: list[Callable[[Any], object]]

class _PeriodicMonitorThread(Protocol):
    def add_monitoring_function(self, function: Callable[[Hub], object], period: float | None) -> object: ...

class IPeriodicMonitorThread(Interface):
    def add_monitoring_function(function: Callable[[Hub], object], period: float | None) -> object: ...

class IPeriodicMonitorThreadStartedEvent(Interface):
    monitor: IPeriodicMonitorThread

@implementer(IPeriodicMonitorThread)
class PeriodicMonitorThreadStartedEvent:
    ENTRY_POINT_NAME: str
    monitor: _PeriodicMonitorThread
    def __init__(self, monitor: _PeriodicMonitorThread) -> None: ...

class IEventLoopBlocked(Interface):
    greenlet: greenlet_t
    blocking_time: float
    info: Sequence[str]

@implementer(IEventLoopBlocked)
class EventLoopBlocked:
    greenlet: greenlet_t
    blocking_time: float
    info: Sequence[str]
    def __init__(self, greenlet: greenlet_t, blocking_time: float, info: Sequence[str]) -> None: ...

class IMemoryUsageThresholdExceeded(Interface):
    mem_usage: int
    max_allowed: int
    memory_info: pmem

class _AbstractMemoryEvent:
    mem_usage: int
    max_allowed: int
    memory_info: pmem
    def __init__(self, mem_usage: int, max_allowed: int, memory_info: pmem) -> None: ...

@implementer(IMemoryUsageThresholdExceeded)
class MemoryUsageThresholdExceeded(_AbstractMemoryEvent): ...

class IMemoryUsageUnderThreshold(Interface):
    mem_usage: int
    max_allowed: int
    max_memory_usage: int
    memory_info: pmem

@implementer(IMemoryUsageUnderThreshold)
class MemoryUsageUnderThreshold(_AbstractMemoryEvent):
    max_memory_usage: int
    def __init__(self, mem_usage: int, max_allowed: int, memory_info: pmem, max_usage: int) -> None: ...

class IGeventPatchEvent(Interface):
    source: object
    target: object

@implementer(IGeventPatchEvent)
class GeventPatchEvent:
    source: object
    target: object
    def __init__(self, source: object, target: object) -> None: ...

class IGeventWillPatchEvent(IGeventPatchEvent): ...
class DoNotPatch(BaseException): ...

@implementer(IGeventWillPatchEvent)
class GeventWillPatchEvent(GeventPatchEvent): ...

class IGeventDidPatchEvent(IGeventPatchEvent): ...

@implementer(IGeventWillPatchEvent)
class GeventDidPatchEvent(GeventPatchEvent): ...

class IGeventWillPatchModuleEvent(IGeventWillPatchEvent):
    source: ModuleType
    target: ModuleType
    module_name: str
    target_item_names: list[str]

@implementer(IGeventWillPatchModuleEvent)
class GeventWillPatchModuleEvent(GeventWillPatchEvent):
    ENTRY_POINT_NAME: str
    source: ModuleType
    target: ModuleType
    module_name: str
    target_item_names: list[str]
    def __init__(self, module_name: str, source: ModuleType, target: ModuleType, items: list[str]) -> None: ...

class IGeventDidPatchModuleEvent(IGeventDidPatchEvent):
    source: ModuleType
    target: ModuleType
    module_name: str

@implementer(IGeventDidPatchModuleEvent)
class GeventDidPatchModuleEvent(GeventDidPatchEvent):
    ENTRY_POINT_NAME: str
    source: ModuleType
    target: ModuleType
    module_name: str
    def __init__(self, module_name: str, source: ModuleType, target: ModuleType) -> None: ...

class IGeventWillPatchAllEvent(IGeventWillPatchEvent):
    patch_all_arguments: Mapping[str, Any]
    patch_all_kwargs: Mapping[str, Any]
    def will_patch_module(module_name: str) -> bool: ...

class _PatchAllMixin:
    def __init__(self, patch_all_arguments: Mapping[str, Any], patch_all_kwargs: Mapping[str, Any]) -> None: ...
    @property
    def patch_all_arguments(self) -> dict[str, Any]: ...  # safe to mutate, it's a copy
    @property
    def patch_all_kwargs(self) -> dict[str, Any]: ...  # safe to mutate, it's a copy

@implementer(IGeventWillPatchAllEvent)
class GeventWillPatchAllEvent(_PatchAllMixin, GeventWillPatchEvent):
    ENTRY_POINT_NAME: str
    def will_patch_module(self, module_name: str) -> bool: ...

class IGeventDidPatchBuiltinModulesEvent(IGeventDidPatchEvent):
    patch_all_arguments: Mapping[str, Any]
    patch_all_kwargs: Mapping[str, Any]

@implementer(IGeventDidPatchBuiltinModulesEvent)
class GeventDidPatchBuiltinModulesEvent(_PatchAllMixin, GeventDidPatchEvent):
    ENTRY_POINT_NAME: str

class IGeventDidPatchAllEvent(IGeventDidPatchEvent): ...

@implementer(IGeventDidPatchAllEvent)
class GeventDidPatchAllEvent(_PatchAllMixin, GeventDidPatchEvent):
    ENTRY_POINT_NAME: str



from _typeshed import Incomplete

class OAuthGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def create_token_from_code(self, params): ...
    def create_token_from_refresh_token(self, params): ...
    def revoke_access_token(self, access_token): ...
    def connect_url(self, raw_params): ...
